/**
 * @file tkl_ota.h
 * @brief Common process - ota
 * @version 1.0.0
 * @date 2021-04-21
 *
 * @copyright Copyright 2018-2021 Tuya Inc. All Rights Reserved.
 *
 */

#ifndef __TKL_OTA_H__
#define __TKL_OTA_H__

#ifdef __cplusplus
	extern "C" {
#endif

#include "tuya_cloud_types.h"


/*
 * ota upgrade type
 */
typedef enum {
    TKL_OTA_AIR_FULL_UPGRADE = 0,   ///< full upgrade via a air transmission
    TKL_OTA_AIR_DIFF_UPGRADE,       ///< diff upgrade via a air transmission
    TKL_OTA_WIRED_FULL_UPGRADE,     ///< full upgrade via a wired transmission
    TKL_OTA_WIRED_AIR_DIFF_UPGRADE, ///< diff upgrade via a wired transmission
} TKL_OTA_TYPE_E;

/*
 * ota download result
 */
typedef enum {
    TKL_OTA_DOWNLOAD_AND_VERIFY_SUCCESS = 0, ///< OTA successful
    TKL_OTA_DOWNLOAD_TIME_OUT           = 1, ///< OTA timeout
    TKL_OTA_VERIFY_FAILED               = 2, ///< OTA image validation failed
    TKL_OTA_SERVER_ABORTED              = 3, ///< An OTA server side exception
    TKL_OTA_CLIENT_ABORTED              = 4, ///< An OTA client side exception
    TKL_OTA_ERASE_FAILED                = 5, ///< Erase flash exceptions
} TKL_OTA_RESULT_E;

/*
 *  ota pack data
 */
typedef struct {
    UINT_T   total_len;     ///< ota image totle len      
    UINT_T   offset;        ///< ota image offset 
    UCHAR_T* data;          ///< ota data
    UINT_T   len;           ///< ota data len
    VOID_T*  pri_data;      ///< private pointer
} TKL_OTA_DATA_T;

/**
* @brief ota start notify
*
* @param[in] image_size:  image size
* @param[in] type:        ota type
*
* @note This API is used for ota start notify
*
* @return OPRT_OK on success. Others on error, please refer to tuya_error_code.h
*/
OPERATE_RET tkl_ota_start_notify(UINT_T image_size, TKL_OTA_TYPE_E type);

/**
* @brief ota data process
*
* @param[in] pack:       point to ota pack
* @param[in] remain_len: ota pack remain len
*
* @note This API is used for ota data process
*
* @return OPRT_OK on success. Others on error, please refer to tuya_error_code.h
*/
OPERATE_RET tkl_ota_data_process(TKL_OTA_DATA_T *pack, UINT_T* remain_len);

/**
* @brief ota end notify
*
* @param[in] reset:  ota reset
*
* @note This API is used for ota end notify
*
* @return OPRT_OK on success. Others on error, please refer to tuya_error_code.h
*/
OPERATE_RET tkl_ota_end_notify(TKL_OTA_RESULT_E reset);


#ifdef __cplusplus
}
#endif

#endif

