/**
* @file tkl_sleep.h
* @brief Common process - adapter the sleep manage api
* @version 0.1
* @date 2021-08-18
*
* @copyright Copyright 2020-2021 Tuya Inc. All Rights Reserved.
*
*/
#ifndef __TKL_SLEEP_H__
#define __TKL_SLEEP_H__

#include "tuya_cloud_types.h"

#ifdef __cplusplus
extern "C" {
#endif


/**
 * @brief cpu sleep mode
 */
typedef enum {
    TKL_CPU_SLEEP,
    TKL_CPU_DEEP_SLEEP,
} TKL_CPU_SLEEP_MODE_E;


/**
 * @brief pre-sleep callback
 */
typedef VOID_T (*TKL_PRE_SLEEP_CB)(VOID_T);

/**
 * @brief post-wakeup callback
 */
typedef VOID_T (*TKL_POST_WAKEUP_CB)(VOID_T);

/**
 * @brief sleep callback
 */
typedef struct {
    TKL_PRE_SLEEP_CB pre_sleep_cb;
    TKL_POST_WAKEUP_CB post_wakeup_cb;
} TKL_SLEEP_CB_T;


/**
 * @brief sleep callback register
 * 
 * @param[in] sleep_cb:  sleep callback
 *
 * @return OPRT_OK on success. Others on error, please refer to tuya_error_code.h
 */
OPERATE_RET tkl_cpu_sleep_callback_register(TKL_SLEEP_CB_T *sleep_cb);

/**
 * @brief allow to sleep
 * 
 * @param[in] none
 *
 * @return none
 */
VOID_T tkl_cpu_allow_sleep(VOID_T);

/**
 * @brief force wakeup
 * 
 * @param[in] none
 *
 * @return none
 */
VOID_T tkl_cpu_force_wakeup(VOID_T);

/**
* @brief Set the low power mode of CPU
*
* @param[in] enable: enable switch
* @param[in] mode:   cpu sleep mode
*
* @note This API is used for setting the low power mode of CPU.
*
* @return OPRT_OK on success. Others on error, please refer to tuya_error_code.h
*/
OPERATE_RET tkl_cpu_sleep_mode_set(BOOL_T enable, TKL_CPU_SLEEP_MODE_E mode);



#ifdef __cplusplus
}
#endif

#endif

