/*
 * Copyright (C) 2021 Tuya Inc.
 */

#include <stdarg.h>
#include <stdio.h>
#include <errno.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <sys/reboot.h>
#include <pthread.h>
#include <linux/rtc.h>
#include <sys/ioctl.h>
#include <sys/time.h>
#include <sys/types.h>
#include <time.h>
#include "hal_log.h"
#include "hal_types.h"
#include "hal_file.h"

#include "tkl_rtc.h"
#include "tkl_rtc_core.h"

#define RTC_DEFAULT_DEVICE  "/dev/rtc0"

static int g_rtc_fd = -1;

OPERATE_RET tkl_rtc_init_common(VOID_T)
{
    char dev_name[MAX_PATH] = {0};
    int ret;

    strncpy(dev_name, RTC_DEFAULT_DEVICE, strlen(RTC_DEFAULT_DEVICE) + 1);

    g_rtc_fd = open(dev_name, O_RDWR);
    if (g_rtc_fd < 0) {
        hal_log(HAL_LOG_ERR, "Failed to open %s\n", dev_name);
        return OPRT_COM_ERROR;
    }

    return OPRT_OK;
}

OPERATE_RET tkl_rtc_deinit_common(VOID_T)
{
    if (g_rtc_fd != -1) {
        close(g_rtc_fd);
        g_rtc_fd = -1;
    }

    return OPRT_OK;
}

OPERATE_RET tkl_rtc_time_set_common(TIME_T time_sec)
{
    time_t timestamp = (time_t)time_sec;
    struct tm tm_utc;
    struct rtc_time rtctime = {0};
    int ret;

    if (gmtime_r(&timestamp, &tm_utc) == NULL) {
        hal_log(HAL_LOG_ERR, "gmtime_r failed\n");
        return OPRT_COM_ERROR;
    }
//    hal_log(HAL_LOG_DEBUG, "set to: %04d-%02d-%02d %02d:%02d:%02d\n",
//            tm_utc.tm_year + 1900, tm_utc.tm_mon + 1, tm_utc.tm_mday,
//            tm_utc.tm_hour, tm_utc.tm_min, tm_utc.tm_sec);

    rtctime.tm_year = tm_utc.tm_year;
    rtctime.tm_mon  = tm_utc.tm_mon;
    rtctime.tm_mday = tm_utc.tm_mday;
    rtctime.tm_hour = tm_utc.tm_hour;
    rtctime.tm_min  = tm_utc.tm_min;
    rtctime.tm_sec  = tm_utc.tm_sec;
    ret = ioctl(g_rtc_fd, RTC_SET_TIME, &rtctime);
    if (ret < 0) {
        hal_log(HAL_LOG_ERR, "RTC_SET_TIME failed\n");
        return OPRT_COM_ERROR;
    }

    return OPRT_OK;
}

OPERATE_RET tkl_rtc_time_get_common(TIME_T *time_sec)
{
    struct rtc_time rtctime;
    struct tm tm_utc = {0};
    time_t timestamp;
    int ret;

    if (time_sec == NULL) {
        return OPRT_INVALID_PARM;
    }

    ret = ioctl(g_rtc_fd, RTC_RD_TIME, &rtctime);
    if (ret < 0) {
        hal_log(HAL_LOG_ERR, "RTC_RD_TIME failed\n");
        return OPRT_COM_ERROR;
    }

    tm_utc.tm_year = rtctime.tm_year;
    tm_utc.tm_mon = rtctime.tm_mon;
    tm_utc.tm_mday = rtctime.tm_mday;
    tm_utc.tm_hour = rtctime.tm_hour;
    tm_utc.tm_min = rtctime.tm_min;
    tm_utc.tm_sec = rtctime.tm_sec;

    timestamp = timegm(&tm_utc);
    if (timestamp == (time_t) -1) {
        hal_log(HAL_LOG_ERR, "timegm failed\n");
        return OPRT_COM_ERROR;
    }

    *time_sec = (TIME_T)timestamp;
    return OPRT_OK;
}
