/**
 * @file utils.c
 * @brief the default weak implements of wired driver, this implement only used when OS=linux
 * @version 0.1
 * @date 2021-10-27
 * 
 * @copyright Copyright 2020-2021 Tuya Inc. All Rights Reserved.
 * 
 */
#include <string.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <net/if.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <pthread.h>
#include <linux/sockios.h>

static char *net_utils_space_trim( char *str )
{
    int i = 0;  
    int count = 0;
    int size = 0;

    char *tmp  = NULL;
    if ( !str ) {
        return NULL;
    }

    size = strlen( str );
    tmp = str;
    
    while ( *str == ' ' || (unsigned char)(*str - 9) <= (13 - 9) ) {
        str++;        
        count++;
    }
    
    if ( count ) {
        memmove( tmp, str, size - count );
        memset( tmp + ( size - count ), 0x0 , size - count );
    }
      
    return (char *) str;    
}

//only find first devname
int net_utils_get_devname( char *devname, int size )
{

    FILE *fp = NULL;
    char *fpath="/proc/net/dev";
    char buf[512];
    int ret = -1;
    char ifname[IFNAMSIZ];
    struct ifreq ifr; 
    char *tmp = NULL;


    if ( !devname || size <= 0 ) {
        printf( "%s %d: arg err\n", __func__, __LINE__ );
        return -1;
    }
 
    fp = fopen( fpath, "r");
    if ( !fp ) {
        printf( "%s %d: %s is not exist\n", __func__, __LINE__, fpath );
        return -1;
    }

    memset( devname, 0x0, size );
    fgets( buf, sizeof( buf ), fp ); 
    fgets( buf, sizeof( buf ), fp );
 
    while ( fgets( buf, sizeof( buf ), fp) ) {
        sscanf( buf, "%[^:]:", ifname );
        
        tmp = net_utils_space_trim( ifname );
        if ( !tmp ) {
            printf( "%s %d: %s error\n", __func__, __LINE__, ifname );
            continue;
        } 
        else {
            //printf( "%s %d: found %s\n", __func__, __LINE__, ifname );
            if ( strncmp( ifname, "en" , 2) == 0 || strncmp( ifname, "eth" , 3) == 0 ) {
                memcpy( devname, ifname, size );
                ret = 0;
                break;
            }
        }

    }

    if ( fp ) {
        fclose( fp );
    }

    return ret;
}

int net_utils_check_cable( const char *ifname, int *plugged )
{

    int sk = -1;
    struct ifreq ifr; 
    int flag = IFF_UP;
    int ret = -1;

    if ( !ifname ) {
        printf( "%s %d: ifname is null\n", __func__, __LINE__ );
        return -1;
    }

    sk = socket(AF_INET, SOCK_DGRAM, 0);
    if ( sk < 0 ) {
        printf( "%s %d: failed to create sk\n", __func__, __LINE__ );
        return -1;
    }

    memset( &ifr, 0, sizeof( ifr ) );
    strncpy( ifr.ifr_name, ifname, sizeof(ifr.ifr_name) - 1 );

    ret = ioctl( sk, SIOCGIFFLAGS, &ifr );
    if ( ret < 0 ) {
        printf( "%s %d: failed to get iterface info \n", __func__, __LINE__ );
        goto exit_end;

    }

    if ( ifr.ifr_flags & IFF_RUNNING ) {
        //printf( "%s %d: %s running \n", __func__, __LINE__, ifname );
        *plugged = 1;
        
    }
    else {
        //printf( "%s %d: %s not running \n", __func__, __LINE__, ifname );
        *plugged = 0;
    }

exit_end:
    if ( sk ) {
        close( sk );
    }
    return ret;
}




