/*********************************************************************************
  *Copyright(C),2015-2021, 
  *TUYA 
  *www.tuya.com
  *FileName:    tuya_ipc_alsa_play_demo.c
**********************************************************************************/
#include "tuya_ipc_common_demo.h"
#if defined(DEMO_IPC_VOICE_ENABLE) && (DEMO_IPC_VOICE_ENABLE == 1)
#include <stdio.h>
#include <math.h>
#include "tuya_ipc_alsa_play_demo.h"

/* set alsa hw params */
STATIC INT_T ty_alsa_hw_params_set(ty_alsa_play_handle_s *play_handle, ty_media_wav_s *wav)
{
    snd_pcm_hw_params_t *hwparams = NULL;
    snd_pcm_format_t pcm_format;
    snd_pcm_uframes_t buffer_size, period_size;
    UINT_T rate;
    UINT_T buffer_time = 0;
    UINT_T period_time = 0;
    INT_T ret;

    if (NULL == play_handle || NULL == wav)
    {
        return -1;
    }

    /* allocate memory for alsa parameters */
    ret = snd_pcm_hw_params_malloc(&hwparams);
    if (ret < 0)
    {
        PR_ERR("malloc pcm hw params failed\n");
        return -1;
    }

    /* choose all parameters */
    ret = snd_pcm_hw_params_any(play_handle->handle, hwparams);
    if (ret < 0)
    {
        PR_ERR("no pcm configurations available");
        snd_pcm_hw_params_free(hwparams);
        return -1;
    }

    /* set the selected read/write foramt */
    ret = snd_pcm_hw_params_set_access(play_handle->handle, hwparams, SND_PCM_ACCESS_RW_INTERLEAVED);
    if (ret < 0)
    {
        PR_ERR("set pcm access type failed");
        snd_pcm_hw_params_free(hwparams);
        return -1;
    }

    /* set the sample format */
    if (wav->format.bits_per_sample == 8)
    {
        pcm_format = SND_PCM_FORMAT_S8;
    }
    else if (wav->format.bits_per_sample == 16)
    {
        pcm_format = SND_PCM_FORMAT_S16_LE;
    }
    else if (wav->format.bits_per_sample == 24)
    {
        pcm_format = SND_PCM_FORMAT_S24_LE;
    }
    else if (wav->format.bits_per_sample == 32)
    {
        pcm_format = SND_PCM_FORMAT_S32_LE;
    }
    else
    {
        PR_ERR("unknow pcm format");
        snd_pcm_hw_params_free(hwparams);
        return -1;
    }
    ret = snd_pcm_hw_params_set_format(play_handle->handle, hwparams, pcm_format);
    if (ret < 0)
    {
        PR_ERR("set pcm sample format failed");
        snd_pcm_hw_params_free(hwparams);
        return -1;
    }
    play_handle->bytes_per_sample = wav->format.bits_per_sample / 8;

    /* set the number of channels */
    ret = snd_pcm_hw_params_set_channels(play_handle->handle, hwparams, wav->format.channels);
    if (ret < 0)
    {
        PR_ERR("set pcm channels failed");
        snd_pcm_hw_params_free(hwparams);
        return -1;
    }
    play_handle->channels = wav->format.channels;

    /* set the sample rate */
    rate = wav->format.samples_per_sec;
    ret = snd_pcm_hw_params_set_rate_near(play_handle->handle, hwparams, &rate, 0);
    if (ret < 0)
    {
        PR_ERR("set pcm rate failed");
        snd_pcm_hw_params_free(hwparams);
        return -1;
    }
    play_handle->rate = rate;

    /* get buffet time max */
    ret = snd_pcm_hw_params_get_buffer_time_max(hwparams, &buffer_time, 0);
    if (buffer_time > 500000 || buffer_time <= 0)
    {
        /* 500ms */
        buffer_time = 500000;
    }

    /* set period time */
    period_time = buffer_time / 4;
    ret = snd_pcm_hw_params_set_period_time_near(play_handle->handle, hwparams, &period_time, 0);
    if (ret < 0)
    {
        PR_ERR("set pcm period time failed");
        snd_pcm_hw_params_free(hwparams);
        return -1;
    }

    /* set buffer time */
    ret = snd_pcm_hw_params_set_buffer_time_near(play_handle->handle, hwparams, &buffer_time, 0);
    if (ret < 0)
    {
        PR_ERR("set pcm buffer time failed");
        snd_pcm_hw_params_free(hwparams);
        return -1;
    }

    /* set hw params */
    ret = snd_pcm_hw_params(play_handle->handle, hwparams);
    if (ret < 0)
    {
        PR_ERR("set pcm hw params failed");
        snd_pcm_hw_params_free(hwparams);
        return -1;
    }

    /* get period size */
    ret = snd_pcm_hw_params_get_period_size(hwparams, &period_size, 0);
    if (ret < 0)
    {
        PR_ERR("get pcm period size failed");
        snd_pcm_hw_params_free(hwparams);
        return -1;
    }
    play_handle->period_size = period_size;
    play_handle->period_data_size = period_size * play_handle->channels * play_handle->bytes_per_sample;
    PR_DEBUG("period size: %d", play_handle->period_size);

    /* get buffer size */
    ret = snd_pcm_hw_params_get_buffer_size(hwparams, &buffer_size);
    if (ret < 0)
    {
        PR_ERR("get pcm buffer size failed");
        snd_pcm_hw_params_free(hwparams);
        return -1;
    }

    snd_pcm_hw_params_free(hwparams);
    return 0;
}

/* set alsa sw params*/
STATIC INT_T ty_alsa_sw_params_set(ty_alsa_play_handle_s *play_handle, ty_media_wav_s *wav)
{
    snd_pcm_sw_params_t *swparams = NULL;
    snd_pcm_uframes_t start_threshold;
    INT_T ret;

    if (NULL == play_handle || NULL == wav)
    {
        return -1;
    }

    /* allocate memory for alsa parameters */
    ret = snd_pcm_sw_params_malloc(&swparams);
    if (ret < 0)
    {
        PR_ERR("malloc pcm sw params failed");
        return -1;
    }

    /* choose software params */
    ret = snd_pcm_sw_params_current(play_handle->handle, swparams);
    if (ret < 0)
    {
        PR_ERR("pcm sw params current failed");
        snd_pcm_sw_params_free(swparams);
        return -1;
    }

    /* set minimum avail size */
    ret = snd_pcm_sw_params_set_avail_min(play_handle->handle, swparams, play_handle->period_size);
    if (ret < 0)
    {
        PR_ERR("set pcm avail min failed");
        snd_pcm_sw_params_free(swparams);
        return -1;
    }

    /* set start_threshold */
    start_threshold = play_handle->period_size * 4;
    ret = snd_pcm_sw_params_set_start_threshold(play_handle->handle, swparams, start_threshold);
    if (ret < 0)
    {
        PR_ERR("set pcm start threshold failed");
        snd_pcm_sw_params_free(swparams);
        return -1;
    }

    /* set sw params */
    ret = snd_pcm_sw_params(play_handle->handle, swparams);
    if (ret < 0)
    {
        PR_ERR("set pcm sw params failed");
        snd_pcm_sw_params_free(swparams);
        return -1;
    }

    snd_pcm_sw_params_free(swparams);
    return 0;
}

/* init alsa play interface */
ty_alsa_play_handle_s *ty_alsa_play_init(ty_media_wav_s *wav)
{
    ty_alsa_play_handle_s *play_handle = NULL;    

    if (NULL == wav)
    {
        PR_ERR("input wav is NULL");
        return NULL;
    }

    play_handle = (ty_alsa_play_handle_s *)malloc(sizeof(ty_alsa_play_handle_s));
    if (NULL == play_handle)
    {
        PR_ERR("malloc play handle failed");
        return NULL;
    }

    /* open alsa interface */
    if (snd_pcm_open(&play_handle->handle, "default", SND_PCM_STREAM_PLAYBACK, 0) < 0)
    {
        PR_ERR("open pcm device failed");
        free(play_handle);
        return NULL;
    }

    /* set alsa hw params */
    if (ty_alsa_hw_params_set(play_handle, wav) != 0)
    {
        PR_ERR("pcm hw params set failed");
        ty_alsa_play_uninit(&play_handle);
        return NULL;
    }

    /* set alsa sw params */
    if (ty_alsa_sw_params_set(play_handle, wav) != 0)
    {
        PR_ERR("pcm sw params set failed");
        ty_alsa_play_uninit(&play_handle);
        return NULL;
    }

    return play_handle;
}

/* uninit alsa play interface */
VOID ty_alsa_play_uninit(ty_alsa_play_handle_s **play_handle)
{
    ty_alsa_play_handle_s *tmp_handle = NULL;

    if (NULL == play_handle)
    {
        return;
    }

    tmp_handle = *play_handle;
    if (NULL == tmp_handle)
    {
        return;
    }

    /* close alsa interface */
    if (NULL != tmp_handle->handle)
    {
        snd_pcm_close(tmp_handle->handle);
    }

    free(tmp_handle);
    *play_handle = NULL;

    return;
}

STATIC INT_T ty_alsa_xrun_recovery(ty_alsa_play_handle_s *play_handle, INT_T err)
{
    if (err == -EPIPE)
    {
        err = snd_pcm_prepare(play_handle->handle);
    }
    else
    {
        PR_ERR("pcm write failed\n");
    }

    return err;
}

/* play data by alsa interface */
INT_T ty_alsa_play_data(ty_alsa_play_handle_s *play_handle, CHAR_T *pData, INT_T len)
{
    INT_T frames_remains = 0;
    INT_T frames = 0;
    INT_T frames_send = 0;
    INT_T ret;

    if (NULL == pData || len <= 0)
    {
        return -1;
    }

    frames_remains = len / (play_handle->channels * play_handle->bytes_per_sample);
    while (frames_remains > 0)
    {
        if (frames_remains > play_handle->period_size)
        {
            frames = play_handle->period_size;
        }
        else
        {
            frames = frames_remains;
        }

        /* write data to alsa driver */
        frames_send = snd_pcm_writei(play_handle->handle, pData, frames);
        if (frames_send < 0)
        {
            ret = ty_alsa_xrun_recovery(play_handle, frames_send);
            if (ret < 0)
            {
                return ret;
            }
        }
        else
        {
            frames_remains -= frames_send;
        }
    }

    return 0;
}

/* set volume by alsa interface */
INT_T ty_alsa_set_volume(INT_T volume)
{
    snd_mixer_t *mixer_handle = NULL;
    snd_mixer_elem_t *elem;
    INT_T volLeft, volRight;
    INT_T volume_db = 0;
    INT_T ret;

    if (volume < 0 || volume > 100)
    {
        PR_ERR("volume out of range");
        return -1;
    }

    /* change linear volume to decibel volume */
    if (volume == 0)
    {
        volume_db = 0;
    }
    else
    {
        volume_db = 100 + 25 * log10(volume * 1.0 / 100.0);
    }
    PR_DEBUG("volume_db: %d", volume_db);

    /* open mixer interface */
    ret = snd_mixer_open(&mixer_handle, 0);
    if (ret < 0)
    {
        PR_ERR("open mixer device failed");
        return -1;
    }

    ret = snd_mixer_attach(mixer_handle, "default");
    if (ret < 0)
    {
        PR_ERR("attach mixer device failed");
        snd_mixer_close(mixer_handle);
        return -1;
    }

    ret = snd_mixer_selem_register(mixer_handle, NULL, NULL);
    if (ret < 0)
    {
        PR_ERR("register mixer selem failed");
        snd_mixer_close(mixer_handle);
        return -1;
    }

    ret = snd_mixer_load(mixer_handle);
    if (ret < 0)
    {
        PR_ERR("load mixer failed");
        snd_mixer_close(mixer_handle);
        return -1;
    }

    elem = snd_mixer_first_elem(mixer_handle);
    while (elem)
    {
        if (strcmp("Master", snd_mixer_selem_get_name(elem)) == 0)
        {
            break;
        }
        elem = snd_mixer_elem_next(elem);
    }

    if (elem == NULL)
    {
        PR_ERR("find Master volume mixer failed");
        snd_mixer_close(mixer_handle);
        return -1;
    }

    /* set playback volume range */
    ret = snd_mixer_selem_set_playback_volume_range(elem, 0, 100);
    if (ret < 0)
    {
        PR_ERR("set volume range failed");
        snd_mixer_close(mixer_handle);
        return -1;
    }

    ret = snd_mixer_handle_events(mixer_handle);
    if (ret < 0)
    {
        PR_ERR("handle mixer events failed");
        snd_mixer_close(mixer_handle);
        return -1;
    }

    /* set left channel volume */
    ret = snd_mixer_selem_set_playback_volume(elem, SND_MIXER_SCHN_FRONT_LEFT, volume_db);
    if (ret < 0)
    {
        PR_ERR("set playback volume failed");
        snd_mixer_close(mixer_handle);
        return -1;
    }

    /* set right channel volume */
    ret = snd_mixer_selem_set_playback_volume(elem, SND_MIXER_SCHN_FRONT_RIGHT, volume_db);
    if (ret < 0)
    {
        PR_ERR("set playback volume failed");
        snd_mixer_close(mixer_handle);
        return -1;
    }

    /* close mixer interface */
    snd_mixer_close(mixer_handle);
    return 0;
}
#endif
