/*********************************************************************************
  *Copyright(C),2015-2020, TUYA www.tuya.comm
  *FileName:    tuya_ipc_p2p_demo
**********************************************************************************/
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>
#include <sys/time.h>
#include <sys/stat.h>

#include "tuya_ipc_common_demo.h"
#include "tuya_ipc_media.h"
#ifdef SUPPORT_ALBUM
#include "tuya_ipc_album.h"
#endif
#ifdef SUPPORT_SWEEPER
#include "tuya_ipc_sweeper_demo.h"
#endif
#include "tuya_ipc_stream_storage.h"
#include "tuya_ipc_p2p.h"
#include "tuya_ipc_media_adapter.h"
#include "tuya_ipc_media_stream_event.h"
#include "tuya_ipc_media_stream.h"
#include "tuya_ipc_p2p_demo.h"
#include "tuya_ipc_media_demo.h"
#include "mem_pool.h"

typedef struct
{
    BOOL_T enabled;
    TRANSFER_VIDEO_CLARITY_TYPE_E live_clarity;
    UINT_T max_users;
    TUYA_CODEC_ID_E p2p_audio_codec;
}TUYA_APP_P2P_MGR;

STATIC TUYA_APP_P2P_MGR s_p2p_mgr = {0};

STATIC VOID __TUYA_APP_media_frame_TO_trans_video(IN CONST MEDIA_FRAME_T *p_in, INOUT MEDIA_VIDEO_FRAME_T *p_out)
{
    UINT_T codec_type = 0;
    codec_type = (p_in->type & 0xff00) >> 8;
    p_out->video_codec = (codec_type == 0 ? TUYA_CODEC_VIDEO_H264 : TUYA_CODEC_VIDEO_H265);
    p_out->video_frame_type = (p_in->type && 0xff) == E_VIDEO_PB_FRAME ? TUYA_VIDEO_FRAME_PBFRAME:TUYA_VIDEO_FRAME_IFRAME;
    p_out->p_video_buf = p_in->p_buf;
    p_out->buf_len = p_in->size;
    p_out->pts = p_in->pts;
    p_out->timestamp = p_in->timestamp;

    return;
}

STATIC VOID __TUYA_APP_media_frame_TO_trans_audio(IN CONST MEDIA_FRAME_T *p_in, INOUT MEDIA_AUDIO_FRAME_T *p_out)
{
    DEVICE_MEDIA_INFO_T media_info = {0};
    tuya_ipc_media_adapter_get_media_info(0, 0, &media_info);
    p_out->audio_codec = media_info.av_encode_info.audio_codec[E_IPC_STREAM_AUDIO_MAIN];
    p_out->audio_sample = media_info.av_encode_info.audio_sample[E_IPC_STREAM_AUDIO_MAIN];
    p_out->audio_databits = media_info.av_encode_info.audio_databits[E_IPC_STREAM_AUDIO_MAIN];
    p_out->audio_channel = media_info.av_encode_info.audio_channel[E_IPC_STREAM_AUDIO_MAIN];
    p_out->p_audio_buf = p_in->p_buf;
    p_out->buf_len = p_in->size;
    p_out->pts = p_in->pts;
    p_out->timestamp = p_in->timestamp;

    return;
}

STATIC VOID __TUYA_APP_ss_pb_event_cb(IN UINT_T pb_idx, IN SS_PB_EVENT_E pb_event, IN PVOID_T args)
{
    PR_DEBUG("ss pb rev event: %u %d", pb_idx, pb_event);
    if(pb_event == SS_PB_FINISH)
    {
        tuya_ipc_playback_send_finish(pb_idx);
    }

    return;
}

STATIC VOID __TUYA_APP_ss_pb_get_video_cb(IN UINT_T pb_idx, IN CONST MEDIA_FRAME_T *p_frame)
{
    MEDIA_VIDEO_FRAME_T video_frame = {0};
    __TUYA_APP_media_frame_TO_trans_video(p_frame, &video_frame);
    tuya_ipc_playback_send_video_frame(pb_idx, &video_frame);

    return;
}

STATIC VOID __TUYA_APP_ss_pb_get_audio_cb(IN UINT_T pb_idx, IN CONST MEDIA_FRAME_T *p_frame)
{
    MEDIA_AUDIO_FRAME_T audio_frame = {0};
    __TUYA_APP_media_frame_TO_trans_audio(p_frame, &audio_frame);
    tuya_ipc_playback_send_audio_frame(pb_idx, &audio_frame);

    return;
}

STATIC VOID __depereated_online_cb(IN TRANSFER_ONLINE_E status)
{
    return;
}

/* Callback functions for transporting events */
INT_T __TUYA_APP_p2p_event_cb(IN CONST INT_T device, IN CONST INT_T channel, IN CONST MEDIA_STREAM_EVENT_E event, IN PVOID_T args)
{
    int ret = 0;
    PR_DEBUG("p2p rev event cb=[%d] ", event);
    if(event != MEDIA_STREAM_SPEAKER_START && event != MEDIA_STREAM_SPEAKER_STOP && args == NULL)
    {
        PR_DEBUG("p2p rev event args null ");
        return -1;
    }
    switch (event)
    {
        case MEDIA_STREAM_LIVE_VIDEO_START:
        {
            C2C_TRANS_CTRL_VIDEO_START * parm = (C2C_TRANS_CTRL_VIDEO_START *)args;
            PR_DEBUG("chn[%u] video start",parm->channel);
            break;
        }
        case MEDIA_STREAM_LIVE_VIDEO_STOP:
        {
            C2C_TRANS_CTRL_VIDEO_STOP * parm = (C2C_TRANS_CTRL_VIDEO_STOP *)args;
            PR_DEBUG("chn[%u] video stop",parm->channel);
            break;
        }
        case MEDIA_STREAM_LIVE_AUDIO_START:
        {
            C2C_TRANS_CTRL_AUDIO_START * parm = (C2C_TRANS_CTRL_AUDIO_START *)args;
            PR_DEBUG("chn[%u] audio start",parm->channel);
            break;
        }
        case MEDIA_STREAM_LIVE_AUDIO_STOP:
        {
            C2C_TRANS_CTRL_AUDIO_STOP * parm = (C2C_TRANS_CTRL_AUDIO_STOP *)args;
            PR_DEBUG("chn[%u] audio stop",parm->channel);
            break;
        }
        case MEDIA_STREAM_SPEAKER_START:
        {
            PR_DEBUG("enbale audio speaker");
            TUYA_APP_Enable_Speaker_CB(TRUE);
            break;
        }
        case MEDIA_STREAM_SPEAKER_STOP:
        {
            PR_DEBUG("disable audio speaker");
            TUYA_APP_Enable_Speaker_CB(FALSE);
            break;
        }
        case MEDIA_STREAM_LIVE_LOAD_ADJUST:
        {
            C2C_TRANS_LIVE_LOAD_PARAM_S *quality = (C2C_TRANS_LIVE_LOAD_PARAM_S *)args;
            PR_DEBUG("live quality %d -> %d", quality->curr_load_level, quality->new_load_level);
            break;
        }
        case MEDIA_STREAM_PLAYBACK_LOAD_ADJUST:
        {
            C2C_TRANS_PB_LOAD_PARAM_S *quality= (C2C_TRANS_PB_LOAD_PARAM_S *)args;
            PR_DEBUG("pb idx:%d quality %d -> %d", quality->client_index, quality->curr_load_level, quality->new_load_level);
            break;
        }
        case MEDIA_STREAM_ABILITY_QUERY:
        {
            C2C_TRANS_QUERY_FIXED_ABI_REQ * pAbiReq;
            pAbiReq = (C2C_TRANS_QUERY_FIXED_ABI_REQ *)args;
            pAbiReq->ability_mask = TY_CMD_QUERY_IPC_FIXED_ABILITY_TYPE_VIDEO |
                                    TY_CMD_QUERY_IPC_FIXED_ABILITY_TYPE_SPEAKER |
                                    TY_CMD_QUERY_IPC_FIXED_ABILITY_TYPE_MIC;          
            break;
        }
        case MEDIA_STREAM_PLAYBACK_QUERY_MONTH_SIMPLIFY:
        {
            C2C_TRANS_QUERY_PB_MONTH_REQ *p = (C2C_TRANS_QUERY_PB_MONTH_REQ *)args;
            PR_DEBUG("pb query by month: %d-%d", p->year, p->month);

            OPERATE_RET ret = tuya_ipc_pb_query_by_month(p->ipcChan, p->year, p->month, &(p->day));
            if (OPRT_OK != ret)
            {
                PR_ERR("pb query by month: %d-%d ret:%d", p->year, p->month, ret);
            }

            break;
        }
        case MEDIA_STREAM_PLAYBACK_QUERY_DAY_TS:
        {
            C2C_TRANS_QUERY_PB_DAY_RESP *pquery = (C2C_TRANS_QUERY_PB_DAY_RESP *)args;
            PR_DEBUG("pb_ts query by day: idx[%d]%d-%d-%d", pquery->channel,pquery->year, pquery->month, pquery->day);
            SS_QUERY_DAY_TS_ARR_T *p_day_ts = NULL;
            OPERATE_RET ret = tuya_ipc_pb_query_by_day(pquery->channel,pquery->ipcChan ,pquery->year, pquery->month, pquery->day, &p_day_ts);
            if (OPRT_OK != ret)
            {
                PR_ERR("pb_ts query by day: %d-%d-%d Fail", pquery->channel,pquery->year, pquery->month, pquery->day);
                break;
            }
            if (p_day_ts){
                printf("%s %d count = %d\n",__FUNCTION__,__LINE__,p_day_ts->file_count);
                PLAY_BACK_ALARM_INFO_ARR * pResult = (PLAY_BACK_ALARM_INFO_ARR *)malloc(sizeof(PLAY_BACK_ALARM_INFO_ARR) + p_day_ts->file_count*sizeof(PLAY_BACK_ALARM_FRAGMENT));
                if (NULL == pResult){
                    printf("%s %d malloc failed \n",__FUNCTION__,__LINE__);
                    free(p_day_ts);
                    pquery->alarm_arr = NULL;
                    return 0;
                }

                INT_T i;
                pResult->file_count = p_day_ts->file_count;
                for (i = 0; i < p_day_ts->file_count; i++){
                    pResult->file_arr[i].type = p_day_ts->file_arr[i].type;
                    pResult->file_arr[i].time_sect.start_timestamp = p_day_ts->file_arr[i].start_timestamp;
                    pResult->file_arr[i].time_sect.end_timestamp = p_day_ts->file_arr[i].end_timestamp;
                }
                pquery->alarm_arr = pResult;
                free(p_day_ts);

            }else{
               pquery->alarm_arr = NULL; 
            }
            break;
        }
        case MEDIA_STREAM_PLAYBACK_START_TS:
        {
            /* Client will bring the start time when playback.
            For the sake of simplicity, only log printing is done. */
            C2C_TRANS_CTRL_PB_START *pParam = (C2C_TRANS_CTRL_PB_START *)args;
            PR_DEBUG("PB StartTS idx:%d %u [%u %u]", pParam->channel, pParam->playTime, pParam->time_sect.start_timestamp, pParam->time_sect.end_timestamp);

            SS_FILE_TIME_TS_T pb_file_info;
            int ret;
            memset(&pb_file_info, 0x00, sizeof(SS_FILE_TIME_TS_T));
            //memcpy(&pb_file_info, &pParam->time_sect, sizeof(SS_FILE_TIME_TS_T));
            pb_file_info.start_timestamp = pParam->time_sect.start_timestamp;
            pb_file_info.end_timestamp = pParam->time_sect.end_timestamp;
            ret = tuya_ipc_ss_pb_start(pParam->channel, __TUYA_APP_ss_pb_event_cb, __TUYA_APP_ss_pb_get_video_cb, __TUYA_APP_ss_pb_get_audio_cb);
            if (0 != ret){
                printf("%s %d pb_start failed\n",__FUNCTION__,__LINE__);
                tuya_ipc_playback_send_finish(pParam->channel);
            }else{
                if (0 != tuya_ipc_ss_pb_seek(pParam->channel, &pb_file_info, pParam->playTime)){
                    printf("%s %d pb_seek failed\n",__FUNCTION__,__LINE__);
                    tuya_ipc_playback_send_finish(pParam->channel);
                }
            }

            break;
        }
        case MEDIA_STREAM_PLAYBACK_PAUSE:
        {
            C2C_TRANS_CTRL_PB_PAUSE *pParam = (C2C_TRANS_CTRL_PB_PAUSE *)args;
            PR_DEBUG("PB Pause idx:%d", pParam->channel);

            tuya_ipc_ss_pb_set_status(pParam->channel, SS_PB_PAUSE);
            break;
        }
        case MEDIA_STREAM_PLAYBACK_RESUME:
        {
            C2C_TRANS_CTRL_PB_RESUME *pParam = (C2C_TRANS_CTRL_PB_RESUME *)args;
            PR_DEBUG("PB Resume idx:%d", pParam->channel);

            tuya_ipc_ss_pb_set_status(pParam->channel, SS_PB_RESUME);
            break;
        }
        case MEDIA_STREAM_PLAYBACK_MUTE:
        {
            C2C_TRANS_CTRL_PB_MUTE *pParam = (C2C_TRANS_CTRL_PB_MUTE *)args;
            PR_DEBUG("PB idx:%d mute", pParam->channel);

            tuya_ipc_ss_pb_set_status(pParam->channel, SS_PB_MUTE);
            break;
        }
        case MEDIA_STREAM_PLAYBACK_UNMUTE:
        {
            C2C_TRANS_CTRL_PB_UNMUTE *pParam = (C2C_TRANS_CTRL_PB_UNMUTE *)args;
            PR_DEBUG("PB idx:%d unmute", pParam->channel);

            tuya_ipc_ss_pb_set_status(pParam->channel, SS_PB_UN_MUTE);
            break;
        }
        case MEDIA_STREAM_PLAYBACK_STOP:
        {
            C2C_TRANS_CTRL_PB_STOP *pParam = (C2C_TRANS_CTRL_PB_STOP *)args;
            PR_DEBUG("PB Stop idx:%d", pParam->channel);

            tuya_ipc_ss_pb_stop(pParam->channel);
            break;
        }
        case MEDIA_STREAM_LIVE_VIDEO_CLARITY_SET:
        {
            C2C_TRANS_LIVE_CLARITY_PARAM_S *pParam = (C2C_TRANS_LIVE_CLARITY_PARAM_S *)args;
            PR_DEBUG("set clarity:%d", pParam->clarity);
            if((pParam->clarity == TY_VIDEO_CLARITY_STANDARD)||(pParam->clarity == TY_VIDEO_CLARITY_HIGH))
            {
                PR_DEBUG("set clarity:%d OK", pParam->clarity);
                s_p2p_mgr.live_clarity = pParam->clarity;
            }
            break;
        }
        case MEDIA_STREAM_LIVE_VIDEO_CLARITY_QUERY:
        {
            C2C_TRANS_LIVE_CLARITY_PARAM_S *pParam = (C2C_TRANS_LIVE_CLARITY_PARAM_S *)args;
            pParam->clarity = s_p2p_mgr.live_clarity;
            PR_DEBUG("query larity:%d", pParam->clarity);
            break;
        }
        case MEDIA_STREAM_DOWNLOAD_START:
        {
            C2C_TRANS_CTRL_DL_START *pParam = (C2C_TRANS_CTRL_DL_START *)args;
            SS_DOWNLOAD_FILES_TS_T strParm={0};
            strParm.dl_start_time = pParam->downloadStartTime;
            strParm.dl_end_time = pParam->downloadEndTime;
            strParm.file_count = pParam->fileNum;
            int len = pParam->fileNum*sizeof(SS_FILE_INFO_T);
            strParm.p_file_info_arr = (SS_FILE_INFO_T*)malloc(len);
            if(strParm.p_file_info_arr== NULL)
            {
                PR_DEBUG("mallocTRANS_DOWNLOAD_START failed");
                break;
            }
            if(pParam->pFileInfo)
            {
                memcpy(strParm.p_file_info_arr, pParam->pFileInfo, len);
            }
            else
            {
                free(strParm.p_file_info_arr);
                PR_DEBUG("TRANS_DOWNLOAD_START p_file_info_arr NULL ");
                break;
            }
            if (OPRT_OK == tuya_ipc_ss_donwload_pre(pParam->channel, &strParm)){
                tuya_ipc_ss_download_set_status(pParam->channel, SS_DL_START);
            }
            free(strParm.p_file_info_arr);
            break;
        }
        case MEDIA_STREAM_DOWNLOAD_STOP:
        {
            C2C_TRANS_CTRL_DL_STOP *pParam = (C2C_TRANS_CTRL_DL_STOP *)args;
            tuya_ipc_ss_download_set_status(pParam->channel, SS_DL_STOP);
            break;
        }
        case MEDIA_STREAM_DOWNLOAD_PAUSE:
        {
            C2C_TRANS_CTRL_DL_PAUSE *pParam = (C2C_TRANS_CTRL_DL_PAUSE *)args;
            tuya_ipc_ss_download_set_status(pParam->channel, SS_DL_PAUSE);
            break;
        }
        case MEDIA_STREAM_DOWNLOAD_RESUME:
        {
            C2C_TRANS_CTRL_DL_RESUME *pParam = (C2C_TRANS_CTRL_DL_RESUME *)args;
            tuya_ipc_ss_download_set_status(pParam->channel, SS_DL_RESUME);
            break;
        }
        case MEDIA_STREAM_DOWNLOAD_CANCLE:
        {
            C2C_TRANS_CTRL_DL_CANCLE *pParam = (C2C_TRANS_CTRL_DL_CANCLE *)args;
            tuya_ipc_ss_download_set_status(pParam->channel, SS_DL_CANCLE);
            break;
        }
        #if 0
        case MEDIA_STREAM_DOWNLOAD_IMAGE:
        {
            C2C_TRANS_CTRL_PB_DOWNLOAD_IMAGE_PARAM_S *pParam = (C2C_TRANS_CTRL_PB_DOWNLOAD_IMAGE_PARAM_S *)args;
            SS_FILE_INFO_T strParm;
            memset(&strParm, 0, sizeof(strParm));
            strParm.start_timestamp = pParam->time_sect.start_timestamp;
            strParm.end_timestamp = pParam->time_sect.end_timestamp;
            printf("TRANS_DOWNLOAD_IMAGE\n");
            if (OPRT_OK !=  tuya_ipc_ss_download_image(pParam->channel, strParm,
                                    &(pParam->image_fileLength), (char **)&(pParam->pBuffer)))
            {
                printf("tuya_ipc_download_image err \n");
            }
            break;
        }
        case MEDIA_STREAM_PLAYBACK_DELETE:
        {
            C2C_TRANS_CTRL_PB_DELDATA_BYDAY_REQ *pParam = (C2C_TRANS_CTRL_PB_DELDATA_BYDAY_REQ *)args;
            if (OPRT_OK !=  tuya_ipc_delete_video(pParam->channel, pParam->year,pParam->month, pParam->day))
            {
                printf("tuya_ipc_ss_delete_video err\n");
            }
            break;
        }
        #endif

#ifdef SUPPORT_ALBUM
        case MEDIA_STREAM_ALBUM_QUERY: /* query album */
        {
            C2C_QUERY_ALBUM_REQ* pSrcType = (C2C_QUERY_ALBUM_REQ*)args;
#ifdef SUPPORT_SWEEPER
            if (0 == strcmp(IPC_SWEEPER_ROBOT, pSrcType->album_name)) {
                ret = tuya_ipc_sweeper_album_cb(event, args);
            }
#else
            ret = tuya_ipc_stor_album_cb(event, args);
#endif
            break;
        }
        case MEDIA_STREAM_ALBUM_DOWNLOAD_START: /* start download album */
        {
            C2C_CMD_IO_CTRL_ALBUM_DOWNLOAD_START* pSrcType = (C2C_CMD_IO_CTRL_ALBUM_DOWNLOAD_START*)args;
#ifdef SUPPORT_SWEEPER
           if (0 == strcmp(IPC_SWEEPER_ROBOT, pSrcType->album_name)) {
                ret = tuya_ipc_sweeper_album_cb(event, args);
            }
#else           
            ret = tuya_ipc_stor_album_cb(event, args);
#endif

            break;
        }
        case MEDIA_STREAM_ALBUM_DOWNLOAD_CANCEL: // cancel album
        {
            C2C_ALBUM_DOWNLOAD_CANCEL* pSrcType = (C2C_ALBUM_DOWNLOAD_CANCEL*)args;
            printf("%s downlaod cancle\n", pSrcType->album_name);
#ifdef SUPPORT_SWEEPER            
            if (0 == strcmp(IPC_SWEEPER_ROBOT, pSrcType->album_name)) {
                ret = tuya_ipc_sweeper_album_cb(event, args);
            }
#else
            ret = tuya_ipc_stor_album_cb(event, args);
#endif            
            break;
        }
        case MEDIA_STREAM_ALBUM_DELETE: //delete
        {
            C2C_CMD_IO_CTRL_ALBUM_DELETE* pSrcType = (C2C_CMD_IO_CTRL_ALBUM_DELETE*)args;
#ifdef SUPPORT_SWEEPER
           if (0 == strcmp(IPC_SWEEPER_ROBOT, pSrcType->album_name)) {
                ret = tuya_ipc_sweeper_album_cb(event, args);
            }
#else           
            ret = tuya_ipc_stor_album_cb(event, args);
#endif
            break;
        }
#endif
        default:
            break;
    }
    return ret;
}

extern CHAR_T s_raw_path[128];
extern BOOL_T enable_tmm_link;


VOID __TUYA_APP_rev_audio_cb(IN INT_T device, IN INT_T channel,IN CONST MEDIA_AUDIO_FRAME_T *p_audio_frame, IN CONST UINT_T frame_no)
{
    MEDIA_FRAME_T audio_frame = {0};
    audio_frame.p_buf = p_audio_frame->p_audio_buf;
    audio_frame.size = p_audio_frame->buf_len;

    printf("Rev Audio. size:[%u] audio_codec:[%d] audio_sample:[%d] audio_databits:[%d] audio_channel:[%d]\n",p_audio_frame->buf_len,
             p_audio_frame->audio_codec, p_audio_frame->audio_sample, p_audio_frame->audio_databits, p_audio_frame->audio_channel);

//    TUYA_APP_Rev_Audio_CB( &audio_frame, TUYA_AUDIO_SAMPLE_8K, TUYA_AUDIO_DATABITS_16, TUYA_AUDIO_CHANNEL_MONO);

    if (enable_tmm_link == TRUE) {
        FILE *fp = NULL;
        CHAR_T media_path[256] = {0};
        sprintf(media_path, "%s/audio.raw", s_raw_path);
        fp = fopen(media_path, "ab+");
        if(fp == NULL){
            printf("fopen failed\n");
            return ;
        }
        fwrite(p_audio_frame->p_audio_buf, p_audio_frame->buf_len, 1, fp);
        fclose(fp);
    }
    return;
}


VOID __TUYA_APP_rev_video_cb(IN INT_T device, IN INT_T channel, IN CONST MEDIA_VIDEO_FRAME_T *p_video_frame)
{
    printf("Rev video. size:[%u] video_codec:[%d] video_frame_type:[%d]\n", p_video_frame->buf_len, p_video_frame->video_codec, p_video_frame->video_frame_type);

    if (enable_tmm_link == TRUE) {
        FILE *fp = NULL;
        CHAR_T media_path[256] = {0};
        sprintf(media_path, "%s/video.h264", s_raw_path);
        fp = fopen(media_path, "ab+");
        if(fp == NULL){
            printf("fopen failed\n");
            return ;
        }
        fwrite(p_video_frame->p_video_buf, p_video_frame->buf_len, 1, fp);
        fclose(fp);
    }
    
    return;
}

VOID __print_file_name(VOID *file_head)
{
    C2C_CMD_IO_CTRL_ALBUM_DOWNLOAD_START *album_head = (C2C_CMD_IO_CTRL_ALBUM_DOWNLOAD_START *)file_head;
    INT_T i = 0;
    printf("print album name:[%s], file cnt:[%d]\n", album_head->albumName, album_head->fileTotalCnt);
    
    for(i = 0; i< album_head->fileTotalCnt; i++) {
        printf("file name [%s]\n", album_head->pFileInfoArr[i].filename);
    }
}

VOID __TUYA_APP_rev_file_cb(IN INT_T device, IN INT_T channel, IN CONST MEDIA_FILE_DATA_T *p_file_data)
{
    if (p_file_data == NULL) {
        printf("file data is NULL\n");
        return;
    }

    if (p_file_data->trans_info == NULL || p_file_data->fragment_info == NULL ) {
        printf("file data is err! [%p][%p]\n", p_file_data->fragment_info, p_file_data->trans_info);
        return;
    }

    STATIC INT_T reqId = 0;
    STATIC FILE *fp = NULL;
    STATIC struct timeval start;
    STATIC struct timeval end;
    C2C_DOWNLOAD_ALBUM_HEAD *head = (C2C_DOWNLOAD_ALBUM_HEAD *)p_file_data->fragment_info;

    if (reqId == 0) {
        reqId = head->reqId;
        __print_file_name(p_file_data->trans_info);
    } else {
    //    printf("reqId:[%d]\n", reqId);
        if (reqId != head->reqId) {
            printf("reqid not match! [%d]-[%d]\n", reqId, head->reqId);
            reqId = 0;
            return; 
        }
    }

    if(p_file_data->status == TY_DATA_TRANSFER_START|| p_file_data->status == TY_DATA_TRANSFER_ONCE)
    {
        printf("new file!!!!\n");
        gettimeofday( &start, NULL );
        char name[32] = {0};
        sprintf(name, "%d", reqId);
        strcpy(name+strlen(name), ".jpg.mid");
        fp = fopen(name, "a+");
        if(fp) {
            printf("open new file:%s, file name:%s\n", name, head->fileName);
        } else {
            printf("open new file fail!!!!\n");
            return;
        }
    }
    
    if(p_file_data->status == TY_DATA_TRANSFER_CANCEL && reqId == head->reqId) {
        //cancel状态，会将cancel的reqid发出来，比较本地的reqid，即可知道对应的文件。可以做删除中间文件的操作
        gettimeofday( &end, NULL );
        printf("cancle file!\n");
        reqId = 0;
        fclose(fp);
        return;
    }
    
    if (fp) {
        fwrite(p_file_data->file_buf, p_file_data->file_len, 1, fp);
    }
    if (p_file_data->status == TY_DATA_TRANSFER_END) {
        fclose(fp);
        
        char old_name[32] = {0};
        char new_name[32] = {0};
        sprintf(old_name, "%d", reqId);
        sprintf(new_name, "%d", reqId);
   
        strcpy(old_name+strlen(old_name), ".jpg.mid");
        strcpy(new_name+strlen(new_name), ".jpg");
        rename(old_name, new_name);
        reqId = 0;
        printf("file success! rename from %s to %s\n", old_name, new_name);
        gettimeofday( &end, NULL );
        struct stat file_stat;
        stat(new_name, &file_stat);
        int timeuse = end.tv_sec - start.tv_sec;
        printf("filesize:%d,Speed:%f MB/s\n", file_stat.st_size, (file_stat.st_size / (float)timeuse) /1024/1024);
    }
      
    return;
}


void demo_test_push_file()
{
    printf("demo_test_push_file\n");

    char snapfile[64];
    snprintf(snapfile,64,"%s/resource/media/demo_snapshot.jpg",s_raw_path);
    FILE*fp = fopen(snapfile,"r+");
    if(NULL == fp)
    {
        printf("fail to open snap.jpg\n");
        return;
    }
    fseek(fp,0,SEEK_END);
    int snapshot_size = ftell(fp);
    char *snapshot_buf = (char *)malloc(snapshot_size);
    fseek(fp,0,SEEK_SET);
    fread(snapshot_buf, snapshot_size, 1, fp);
    fclose(fp);
    TUYA_IPC_BRIEF_FILE_INFO_4_APP strBriefInfo = { 0 };
    strBriefInfo.fileType = E_FILE_TYPE_2_APP_PANORAMA;
    strBriefInfo.param = 1;

    int handle = tuya_ipc_start_send_file_to_app(&strBriefInfo);
    printf("#####tuya_ipc_start_send_file_to_app handle %d \n\n\n\n\n", handle);


    sleep(4);
    printf("sleep over..\n\n\n");

    TUYA_IPC_FILE_INFO_4_APP strfileInfo = { 0 };
    strfileInfo.len = snapshot_size;
    strfileInfo.buff = snapshot_buf;
    strfileInfo.fileName = Malloc(48);
    INT_T i = 0;
    int ret;
    for(i=0; i<6; i++) {
        sprintf(strfileInfo.fileName, "Panorama_%d.jpg", i);
        ret = tuya_ipc_send_file_to_app(handle, &strfileInfo, 30);
        if (0 != ret) {
            printf("push file %s ret %d \n", "test.jpg", ret);
        }
    }

    if (strfileInfo.fileName) {
        Free(strfileInfo.fileName);
    }   

    int ret1 = tuya_ipc_stop_send_file_to_app(handle);  
    printf("tuya_ipc_stop_send_file_to_app ret %d \n", ret1);

    return;
}

VOID IPC_APP_Media_Adapter_Init(TUYA_IPC_SDK_MEDIA_ADAPTER_S *p_media_adatper_info, TUYA_IPC_SDK_MEDIA_STREAM_S *p_media_infos)
{
    TUYA_IPC_MEDIA_ADAPTER_VAR_T media_var;
    media_var.get_snapshot_cb = p_media_adatper_info->get_snapshot_cb;
    media_var.on_recv_audio_cb = p_media_adatper_info->rev_audio_cb;
    media_var.on_recv_video_cb = p_media_adatper_info->rev_video_cb;
    media_var.on_recv_file_cb = p_media_adatper_info->rev_file_cb;
    
    tuya_ipc_media_adapter_init(&media_var);
    
    DEVICE_MEDIA_INFO_T device_media_info = { 0 };
    memcpy(&device_media_info.av_encode_info, &p_media_infos->media_info,sizeof(IPC_MEDIA_INFO_T));
    
    device_media_info.audio_decode_info.enable = 1;
    device_media_info.audio_decode_info.audio_codec = TUYA_CODEC_AUDIO_G711U;
    device_media_info.audio_decode_info.audio_sample = TUYA_AUDIO_SAMPLE_8K;
    device_media_info.audio_decode_info.audio_databits = TUYA_AUDIO_DATABITS_16;
    device_media_info.audio_decode_info.audio_channel = TUYA_AUDIO_CHANNEL_MONO;
    
    device_media_info.max_pic_len = MAX_SNAPSHOT_BUFFER_SIZE;

    tuya_ipc_media_adapter_set_media_info(0, 0, device_media_info);
#if defined(IPC_CHANNEL_NUM) && (IPC_CHANNEL_NUM==2)
    tuya_ipc_media_adapter_set_media_info(0, 1, device_media_info);
#endif    

    return;
}

VOID IPC_APP_Media_Stream_Init(TUYA_IPC_SDK_MEDIA_ADAPTER_S *p_media_adatper_info)
{
    MEDIA_STREAM_VAR_T  media_stream_var = {0};
    media_stream_var.on_event_cb = p_media_adatper_info->media_event_cb;
    media_stream_var.max_client_num = p_media_adatper_info->max_stream_client;
    media_stream_var.def_live_mode = p_media_adatper_info->live_mode;
    int ret = tuya_ipc_media_stream_init(&media_stream_var);
    PR_DEBUG("media stream init result is %d\n",ret);

    return;
}