/*********************************************************************************
  *Copyright(C),2015-2020, TUYA www.tuya.comm
  *FileName:    tuya_ipc_sd_demo
**********************************************************************************/
#include <sys/statfs.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/prctl.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <pthread.h>
#include <dirent.h> 
#include "tuya_ipc_api.h"
#include "tuya_ipc_stream_storage.h"
#include "tuya_ipc_common_demo.h"


#include "tuya_ipc_media.h"
#include "tuya_ipc_p2p.h"
#include "tuya_ipc_stream_storage_demo.h"
#include "tuya_ipc_media_demo.h"
#include "tuya_ipc_dp_handler.h"
#include "tuya_ipc_log_demo.h"

#define SUPPORT_ALBUM
#ifdef SUPPORT_ALBUM
#include "tuya_ipc_album.h"
#endif


/************************
Description: Using the TUYA SD card storage and playback function, 
the developers need to implement the relevant interface.
Note：Interface names cannot be changed, and these interfaces are declared and used in SDK.
This demo file gives the management implementation of SD card operation for typical embedded devices in Linux system.
Developers can modify it according to their practical application.

VOID tuya_ipc_sd_format(VOID);
VOID tuya_ipc_sd_remount(VOID);
SD_STATUS_E tuya_ipc_sd_get_status(VOID);
VOID tuya_ipc_sd_get_capacity(UINT_T *p_total, UINT_T *p_used, UINT_T *p_free);
CHAR_T *tuya_ipc_get_sd_mount_path(VOID);

************************/
#define MAX_MMC_NAME_LEN        16
#define MAX_MOUNTS_INFO_LEN     1024
#define LINUX_SD_DEV_FILE       "/dev/mmcblk0"
#define LINUX_MOUNT_INFO_FILE   "/proc/mounts"
#define SD_MOUNT_PATH           "/tmp" // keep same to IPC_APP_SD_BASE_PATH
#define FORMAT_CMD              "mkfs.vfat"

STATIC CHAR_T s_mounts_info[MAX_MOUNTS_INFO_LEN];
STATIC CHAR_T s_mmcblk_name[MAX_MMC_NAME_LEN] = {0};


//Implementation Interface of Formatting Operation
VOID tuya_ipc_sd_format(VOID)
{
    CHAR_T format_cmd[256] = {0};
    char buffer[512] = {0};
    PR_DEBUG("sd format begin\n");

    snprintf(format_cmd,256,"umount %s;%s %s;mkdir -p /mnt/sdcard;mount -t auto %s /mnt/sdcard;",s_mmcblk_name,FORMAT_CMD,s_mmcblk_name,s_mmcblk_name);
    PR_DEBUG("execute: %s\n",format_cmd);
    FILE *pp = popen(format_cmd, "r");
    if(NULL != pp)
    {
        fgets(buffer,sizeof(buffer),pp);
        printf("%s\n",buffer);
        pclose(pp);
    }
    else
    {
        PR_DEBUG("format_sd_card failed\n");
    }
    PR_DEBUG("sd format end\n");
}

//Implementation Interface for Remounting
VOID tuya_ipc_sd_remount(VOID)
{
    CHAR_T format_cmd[128] = {0};
    char buffer[512] = {0};
    SD_STATUS_E status = SD_STATUS_UNKNOWN;

    status = tuya_ipc_sd_get_status();
    if(SD_STATUS_NORMAL == status)
    {
        PR_DEBUG("sd don't need to remount!\n");
        return;
    }
    PR_DEBUG("remount_sd_card ..... \n");

    snprintf(format_cmd,128,"umount %s;sleep 1;mount -t auto %s /mnt/sdcard;",s_mmcblk_name,s_mmcblk_name);
    FILE *pp = popen(format_cmd, "r");
    if(NULL != pp)
    {
        fgets(buffer,sizeof(buffer),pp);
        printf("%s\n",buffer);
        pclose(pp);
    }
    else
    {
        printf("remount_sd_card failed\n");
    }
}

STREAM_STORAGE_WRITE_MODE_E tuya_ipc_sd_get_mode_config(VOID)
{
    BOOL_T sd_on = IPC_APP_get_sd_record_onoff();
    if(sd_on)
    {
        UINT_T sd_mode = IPC_APP_get_sd_record_mode();
        if(0 == sd_mode)
        {
            return SS_WRITE_MODE_EVENT;
        }
        else
        {
            return SS_WRITE_MODE_ALL;
        }
    }
    else
    {
        return SS_WRITE_MODE_NONE;
    }
}

//Implementation Interface for Obtaining SD Card Status
SD_STATUS_E tuya_ipc_sd_get_status(VOID)
{
	return SD_STATUS_NORMAL;// debugg

    // UserTodo
    FILE *fp = fopen(LINUX_SD_DEV_FILE, "rb");
    if(!fp)
    {
        return SD_STATUS_NOT_EXIST;
    }
    fclose(fp);

    strcpy(s_mmcblk_name,LINUX_SD_DEV_FILE);
    if(0 == access(LINUX_SD_DEV_FILE"p1",F_OK)) //Default node name information
    {
        strcat(s_mmcblk_name,"p1");
    }
    
    fp = fopen(LINUX_MOUNT_INFO_FILE, "rb");
    if(fp)
    {
        memset(s_mounts_info,0,sizeof(s_mounts_info));
        fread(s_mounts_info,1,MAX_MOUNTS_INFO_LEN,fp);
        fclose(fp);
        CHAR_T *mmcblk_name_start = strstr(s_mounts_info,"/dev/mmcblk"); //Confirm the final node name in Mount information
        CHAR_T *mmcblk_name_end = strstr(s_mounts_info," /mnt/sdcard");
        if(mmcblk_name_start && mmcblk_name_end)
        {
            int mmcblk_name_len = mmcblk_name_end-mmcblk_name_start;
            if(mmcblk_name_len >= MAX_MMC_NAME_LEN)
            {
                 return SD_STATUS_ABNORMAL;
            }
            strncpy(s_mmcblk_name, mmcblk_name_start, mmcblk_name_len);
            s_mmcblk_name[mmcblk_name_len] = '\0';
        }
        //There are device nodes but no mount information. Generally, the card format is incorrect and report abnormal.
        else 
        {
            return SD_STATUS_ABNORMAL;
        } 
        //If the mount information of the SD card is not at the end and there is a ro mount behind it, there will be a problem.
        if(NULL != strstr(mmcblk_name_start,"ro,")) 
        {
            return SD_STATUS_ABNORMAL;
        }
        if(NULL == strstr(mmcblk_name_start,"vfat"))
        {
            return SD_STATUS_ABNORMAL;
        }
        if (access(s_mmcblk_name,0))
        {
            return SD_STATUS_ABNORMAL;
        }

        return SD_STATUS_NORMAL;
    }
    else
    {
        return SD_STATUS_UNKNOWN;
    }
}

//SD card capacity acquisition interface, unit: KB
VOID tuya_ipc_sd_get_capacity(UINT_T *p_total, UINT_T *p_used, UINT_T *p_free)
{
    *p_total = 1024*1024;
    *p_used = 1024*20;
    *p_free =  *p_total  - *p_used;
    return ;

    struct statfs sd_fs;
    if (statfs("/mnt/sdcard", &sd_fs) != 0)
    {  
        PR_ERR("statfs failed!/n");
        return;
    }

    *p_total = (UINT_T)(((UINT64_T)sd_fs.f_blocks * (UINT64_T)sd_fs.f_bsize) >> 10);
    *p_used = (UINT_T)((((UINT64_T)sd_fs.f_blocks - (UINT64_T)sd_fs.f_bfree) * (UINT64_T)sd_fs.f_bsize) >> 10);
    *p_free = (UINT_T)(((UINT64_T)sd_fs.f_bavail * (UINT64_T)sd_fs.f_bsize) >> 10);
    PR_DEBUG("sd capacity: total: %d KB, used %d KB, free %d KB\n",*p_total,*p_used,*p_free);
    return;
}

//get the path of mounting sdcard
CHAR_T *tuya_ipc_get_sd_mount_path(VOID)
{
    return SD_MOUNT_PATH;
}

INT_T tuya_ipc_sd_status_upload(INT_T status)
{
	IPC_APP_report_sd_status_changed(status);
	return 0;
}

int tuya_ipc_ss_media_cb(int cameraId, IPC_MEDIA_INFO_T* param)
{
    if (cameraId < 0 || cameraId > 1) {
        return -1;
    }

    DEVICE_MEDIA_INFO_T media_info = {0};
    tuya_ipc_media_adapter_get_media_info(0, cameraId, &media_info);
    memcpy(param, &media_info.av_encode_info, sizeof(IPC_MEDIA_INFO_T));

    return 0;
}

OPERATE_RET TUYA_APP_Init_Stream_Storage(TUYA_IPC_SDK_LOCAL_STORAGE_S * p_local_storage_info)
{
    STATIC BOOL_T s_stream_storage_inited = FALSE;
    if(s_stream_storage_inited == TRUE)
    {
        PR_DEBUG("The Stream Storage Is Already Inited");
        return OPRT_OK;
    }

    if(p_local_storage_info == NULL)
    {
        PR_DEBUG("Init Stream Storage fail. Param is null");
        return OPRT_INVALID_PARM;
    }
    TUYA_IPC_STORAGE_VAR_T stg_var;
    memset(&stg_var, 0, SIZEOF(TUYA_IPC_STORAGE_VAR_T));
    memcpy(stg_var.base_path, p_local_storage_info->storage_path, SS_BASE_PATH_LEN);
    stg_var.max_event_per_day = p_local_storage_info->max_event_num_per_day;
    stg_var.sd_status_changed_cb = p_local_storage_info->sd_status_cb;

    
#if defined(SUPPORT_ALBUM) ||  defined(SUPPORT_ALBUM_EMERGECY) || defined(SUPPOR_PUSH_PANORAMA)
    stg_var.album_info.cnt =1;
    memcpy(&stg_var.album_info.album_name[0], TUYA_IPC_ALBUM_EMERAGE_FILE,strlen(TUYA_IPC_ALBUM_EMERAGE_FILE));
#endif
    PR_DEBUG("Init Stream_Storage SD:%s", p_local_storage_info->storage_path);
    OPERATE_RET ret = tuya_ipc_ss_init(&stg_var);
    if(ret != OPRT_OK)
    {
        PR_ERR("Init Main Video Stream_Storage Fail. %d", ret);
        return OPRT_COM_ERROR;
    }
    return OPRT_OK;
}

/* Callback functions for transporting events */
int tuya_ipc_stor_album_cb(int event, void* args)
{
    PR_DEBUG("p2p rev event cb=[%d] ", event);
    switch (event)
    {
        /* 内容结构：
        C2C_ALBUM_INDEX_HEAD
        C2C_ALBUM_INDEX_ITEM
        C2C_ALBUM_INDEX_ITEM
        C2C_ALBUM_INDEX_ITEM
        */
        case MEDIA_STREAM_ALBUM_QUERY: /* query album */
        {
            C2C_QUERY_ALBUM_REQ* pSrcType = (C2C_QUERY_ALBUM_REQ*)args;

            // NOTICE: pIndexFile malloc/free in SDK
            int ret = tuya_ipc_album_query_by_name(pSrcType->albumName, pSrcType->channel, &pSrcType->fileLen, (C2C_ALBUM_INDEX_HEAD**)&(pSrcType->pIndexFile));
            if (0 != ret) {
                PR_ERR("err %d", ret); 
            }

            if (pSrcType->pIndexFile) {
                C2C_ALBUM_INDEX_HEAD* ptmp = (C2C_ALBUM_INDEX_HEAD*)pSrcType->pIndexFile;
                PR_DEBUG("get album items %d", ptmp->itemCount);
            }
            break;
        }
        case MEDIA_STREAM_ALBUM_DOWNLOAD_START: /* start download album */
        {
            C2C_CMD_IO_CTRL_ALBUM_DOWNLOAD_START* pSrcType = (C2C_CMD_IO_CTRL_ALBUM_DOWNLOAD_START*)args;
            printf("start download\n");
            SS_DOWNLOAD_STATUS_E status = 0;
            SS_ALBUM_DOWNLOAD_START_INFO_T strStarInfo = { 0 };
            strStarInfo.session_id = pSrcType->channel;
            memcpy(strStarInfo.album_name, pSrcType->albumName, strlen(pSrcType->albumName));
            strStarInfo.file_count = pSrcType->fileTotalCnt;
            strStarInfo.thumbnail = pSrcType->thumbnail;
            strStarInfo.p_file_info_arr = (SS_FILE_PATH_T*)pSrcType->pFileInfoArr;
            int ret = tuya_ipc_album_set_download_status(SS_DL_START, &strStarInfo);
            if (0 != ret) {
                PR_ERR("err %d", ret);
            }
            break;
        }
        case MEDIA_STREAM_ALBUM_DOWNLOAD_CANCEL: // cancel album
        {
            // thread do nothing...
            C2C_ALBUM_DOWNLOAD_CANCEL* pSrcType = (C2C_ALBUM_DOWNLOAD_CANCEL*)args;
            SS_ALBUM_DOWNLOAD_START_INFO_T strStarInfo = { 0 };
            strStarInfo.session_id = pSrcType->channel;
            memcpy(strStarInfo.album_name, pSrcType->albumName, strlen(pSrcType->albumName));
            int ret = tuya_ipc_album_set_download_status(SS_DL_CANCLE, &strStarInfo);
            if (0 != ret) {
                PR_ERR("err %d", ret);
            }
            PR_ERR("ok %d", ret);

            break;
        }
        case MEDIA_STREAM_ALBUM_DELETE: //delete
        {
            C2C_CMD_IO_CTRL_ALBUM_DELETE* pSrcType = (C2C_CMD_IO_CTRL_ALBUM_DELETE*)args;
            int ret = tuya_ipc_album_delete_by_file_info(pSrcType->channel, pSrcType->albumName, pSrcType->fileNum, (SS_FILE_PATH_T*)pSrcType->pFileInfoArr);
            if (0 != ret) {
                PR_ERR("err %d", ret);
                tuya_ipc_delete_video_finish_v2(pSrcType->channel, TUYA_DOWNLOAD_ALBUM, 0);
            }
            break;
        }
        default:
            return -1;
            break;
    }
    return 0;
}


#ifdef SUPPORT_ALBUM_EMERGECY

/* album info */
extern CHAR_T s_raw_path[128];
#define ABS_PATH_LEN 128
#define CMD_LEN 1024
static char s_test_pic_thumbNail[ABS_PATH_LEN] = "thumbnail/pic_thumbnail.jpg";
static char s_test_mp4_thumbNail[ABS_PATH_LEN] = "thumbnail/mp4_thumbnail.jpg";
static char s_raw_emergency_path[ABS_PATH_LEN] = { 0 };
static char s_raw_emergency_thumbnail[ABS_PATH_LEN] = { 0 };
static char s_emergency_path[ABS_PATH_LEN] = { 0 };
static char s_emergency_thunbnail[ABS_PATH_LEN] = { 0 };

void album_jpg_file_put(char *jpgFile)
{
    CHAR_T cmd[CMD_LEN];
    int curtime = uni_time_get_posix();
    
    //init target file path
    char targetFile[ABS_PATH_LEN] = { 0 };
    memset(targetFile, 0, sizeof(targetFile));
    sprintf(targetFile, "%d_ch1.jpg", curtime);
    
    //cp pic and thumbnail to emergency album
    memset(cmd, 0, sizeof(cmd));
    sprintf(cmd, "cp %s/%s %s/%s", s_raw_emergency_path, jpgFile, s_emergency_path, targetFile);//TODO put jpg into the path
    system(cmd);
    memset(cmd, 0, sizeof(cmd));
    sprintf(cmd, "cp %s/%s %s/%s", s_raw_emergency_path, s_test_pic_thumbNail, s_emergency_thunbnail, targetFile);//TODO put thumbnail into the path
    system(cmd);

    // write file info
    ALBUM_FILE_INFO strInfo;
    memset(&strInfo, 0, sizeof(ALBUM_FILE_INFO));
    strInfo.channel = 0;
    strInfo.type = SS_DATA_TYPE_PIC;
    memcpy(&strInfo.fileName, targetFile, strlen(targetFile));
    strInfo.createTime = curtime;
    strInfo.duration = 0;
    tuya_ipc_album_write_file_info(TUYA_IPC_ALBUM_EMERAGE_FILE, &strInfo);

    return;
}

void album_mp4_file_put(char *mp4File)
{
    CHAR_T cmd[CMD_LEN];
    int curtime = uni_time_get_posix();

    //init target file path
    char targetFile[ABS_PATH_LEN] = { 0 };
    memset(targetFile, 0, sizeof(targetFile));
    sprintf(targetFile, "%d_ch2.mp4", curtime);
    char mp4Pic[ABS_PATH_LEN] = { 0 };
    memset(mp4Pic, 0, sizeof(mp4Pic));
    sprintf(mp4Pic, "%d_ch2.jpg", curtime);
    // cp video and thumbnail to emergency album
    memset(cmd, 0, sizeof(cmd));
    sprintf(cmd, "cp %s/%s %s/%s", s_raw_emergency_path, mp4File, s_emergency_path, targetFile);//TODO put mp4 into the path
    printf("cmd:%s\n",cmd);
    system(cmd);

    memset(cmd, 0, sizeof(cmd));
    sprintf(cmd, "cp %s/%s %s/%s", s_raw_emergency_path, s_test_mp4_thumbNail, s_emergency_thunbnail, mp4Pic);//TODO put thumbnail into the path
     printf("cmd:%s\n",cmd);
     system(cmd);
    // write file info
    ALBUM_FILE_INFO strInfo;
    memset(&strInfo, 0, sizeof(ALBUM_FILE_INFO));
    strInfo.channel = 1;
    strInfo.type = SS_DATA_TYPE_VIDOE;
    memcpy(&strInfo.fileName, targetFile, strlen(targetFile));
    strInfo.createTime = curtime;
   // char* p = mp4File + strlen("1234567890_");
    strInfo.duration = 50;// todo check atoi(p);
    tuya_ipc_album_write_file_info(TUYA_IPC_ALBUM_EMERAGE_FILE, &strInfo);

    return;
}

/*
  本地相册文件名要求；
  ipc_emergency_record/
        A.mp4
        B.jpg
        thumbnail/
              A.jpg
              B.jpg
  缩略图文件名和源文件文件名必须相同(除后缀外);
  源文件的文件名不可重名;

  本地相册列表交互ver1:
  1.user获取某相册路径、某相册缩略图路径，
  2.user在相册路径写入源数据（mp4/图片）
  3.user在缩略图路径写入对应缩略图
  4.user通过tuya_ipc_album_write_file_info，更新文件信息
*/
 static int emerge_album_path_inited = 0;
/*
  存放紧急相册文件(jpg/mp4文件)
*/

void album_file_init()
{
    if (0 == emerge_album_path_inited) {
        emerge_album_path_inited = 1;
        tuya_ipc_album_get_path(TUYA_IPC_ALBUM_EMERAGE_FILE, s_emergency_path, s_emergency_thunbnail);
        printf("get album file path %s \nthumbNail %s\n", s_emergency_path, s_emergency_thunbnail);

        sprintf(s_raw_emergency_path, "%s/resource/ipc_emergency_record", s_raw_path);
        sprintf(s_raw_emergency_thumbnail, "%s/resource/ipc_emergency_record/thumbnail", s_raw_path);
    }

}
void  album_file_put()
{
    album_file_init();
    CHAR_T cmd[ABS_PATH_LEN];

    char* testSrcFile;
    DIR* p_root_dir = NULL;
    struct dirent* p_child_dir = NULL;
    /* copy mp4/jpg from src file to album path, and write file info */
    p_root_dir = opendir(s_raw_emergency_path);
    if (p_root_dir) {
        while ((p_child_dir = readdir(p_root_dir)) != NULL) {
            if (strcmp(p_child_dir->d_name, ".") == 0
                || strcmp(p_child_dir->d_name, "..") == 0) {
                continue;
            }
            sleep(2);//控制创建速度
            int curtime = uni_time_get_posix();
            testSrcFile = p_child_dir->d_name;
            if (strstr(testSrcFile, "jpg")) {
                album_jpg_file_put(testSrcFile);
            } else if (strstr(testSrcFile, "mp4")) {
                album_mp4_file_put(testSrcFile);
            }
        }
        closedir(p_root_dir);
    }
}

void album_file_put_v2()
{
    album_file_init();

    int startTime = 1624718683;
    int endTime = 1624718683 + 9;
    int ret;
    // conver record to mp4
    SS_FILE_TIME_TS_T file_info;
    file_info.start_timestamp = startTime; // todo user get from query
    file_info.end_timestamp = endTime;
    char mp4_path_prefix[64] = { 0 };
    char targetFile[48] = { 0 };
    memset(mp4_path_prefix, 0, sizeof(mp4_path_prefix));
    // snprintf(mp4_path_prefix, 64, "%s/%dCh0",s_emergency_path, startTime );// 绝对路径，后缀名由sdk补充完整
    //  snprintf(mp4_path_prefix, 64, "%s/%dCh0",s_emergency_path, startTime );// 绝对路径，后缀名由sdk补充完整
    // 1624761220_ch1_0000.mp4 // todo fileName able to get
    ret = tuya_ipc_ss_save_album_mp4_by_chan(0, &file_info, s_emergency_path, targetFile, sizeof(targetFile));
    if (0 != ret) {
        printf("err");
        return -1;
    }

    //   snprintf(targetFile, 48, "%dCh0_0000.mp4",startTime );
    // write file info
    ALBUM_FILE_INFO strInfo;
    memset(&strInfo, 0, sizeof(ALBUM_FILE_INFO));
    strInfo.channel = 1;
    strInfo.type = SS_DATA_TYPE_VIDOE;
    memcpy(&strInfo.fileName, targetFile, strlen(targetFile));
    strInfo.createTime = startTime;
    // char* p = mp4File + strlen("1234567890_");
    strInfo.duration = endTime - startTime; // todo check atoi(p);
    tuya_ipc_album_write_file_info(TUYA_IPC_ALBUM_EMERAGE_FILE, &strInfo);
}
#endif

#ifdef SUPPORT_ALBUM_DEFAULT
#define PanoramaFileName "Panorama.jpg"
void  album_file_put_Panorama()
{
    // write file info
    ALBUM_FILE_INFO strInfo;
    memset(&strInfo, 0, sizeof(ALBUM_FILE_INFO));
    strInfo.channel = 1;
    strInfo.type = SS_DATA_TYPE_PANORAMA;
    memcpy(&strInfo.fileName, PanoramaFileName, strlen(PanoramaFileName));
    strInfo.createTime = uni_time_get_posix();;
    strInfo.duration = 0;
    tuya_ipc_album_write_file_info(TUYA_IPC_ALBUM_DEFAULT, &strInfo);

}
#endif