/*********************************************************************************
  *Copyright(C),2015-2021, 
  *TUYA 
  *www.tuya.com
  *FileName:    tuya_ipc_voice_demo.c
**********************************************************************************/
#include "tuya_ipc_common_demo.h"
#if defined(DEMO_IPC_VOICE_ENABLE) && (DEMO_IPC_VOICE_ENABLE == 1)
#include <stdio.h>
#include <string.h>
#include <pthread.h>
#include <dirent.h>
#include <unistd.h>
#include <sys/prctl.h>
#include "tuya_ipc_dp_utils.h"
#include "tuya_ipc_media_wave_parse.h"
#include "tuya_ipc_speaker_demo.h"
#include "tuya_ipc_voice.h"
#include "tuya_ipc_voice_demo.h"

#define IPC_VOICE_CODE_LEN       64
#define IPC_VOICE_LIST_UPDATE_DURATION 1800
#define IPC_VOICE_DOWNLOAD_RETRY 3
#define IPC_VOICE_DEFAULT_VOLUME 75
#define IPC_VOICE_CODE              "phi-no-bm"
#define IPC_VOICE_FILE_PATH         "/home/share/chuishi/Project/os_5.3/voice"
#define IPC_VOICE_STR_DOWNLOADING   "downloading"
#define IPC_VOICE_STR_DOWNLOAD_FAIL "download failed"
#define IPC_VOICE_STR_PLAY_FAIL     "play failed"

typedef enum {
    IPC_VOICE_STATE_PLAYING              = 0,
    IPC_VOICE_STATE_STOPPING             = 1
} IPC_VOICE_STATE_E;

typedef enum {
    IPC_VOICE_STATUS_IDLE                = 0,
    IPC_VOICE_STATUS_READY               = 1,
    IPC_VOICE_STATUS_DOWNLOAD_START      = 2,
    IPC_VOICE_STATUS_DOWNLOADING         = 3,
    IPC_VOICE_STATUS_PLAY_START          = 4,
    IPC_VOICE_STATUS_PLAYING             = 5,
    IPC_VOICE_STATUS_STOP                = 6
} IPC_VOICE_STATUS_E;

typedef struct{
    BOOL_T              inited;
    IPC_VOICE_STATE_E   state;
    IPC_VOICE_STATUS_E  status;
    pthread_t           stid;
    pthread_mutex_t     lock;

    /* voice list */
    BOOL_T              voice_update;                          // force update voice list
    CHAR_T              voice_code[IPC_VOICE_CODE_LEN+1];      // voice code
    TIME_T              voice_list_update_time;                // voice list update time
    UINT_T              max_voice_num;                         // max number of voice in voice list
    UINT_T              voice_num;                             // voice number in voice list
    VOICE_LIST_S        *voice_list;                           // voice list

    /* downloading parameters */
    BOOL_T              downloading;                           // downloading flag
    INT_T               download_result;                       // download result
    INT_T               download_fail_cnt;                     // downloading fail cnt

    /* play control parameters */
    IPC_VOICE_PLAY_MODE_E   play_mode;                         // play_mode: loop, loop1, shuffle
    INT_T               play_index;                            // playing voice index in voice list
    INT_T               play_id;                               // playing voice id
    CHAR_T              play_filename[IPC_WAV_FILENAME_LEN+1]; // playing filename

    /* file pointer */
    FILE               *fp;
} ty_voice_handle_s;

STATIC ty_voice_handle_s *ty_voice_get_handle()
{
    static ty_voice_handle_s s_voice_hdl = {.lock = PTHREAD_MUTEX_INITIALIZER};

    return &s_voice_hdl;
}

STATIC VOID __tuya_app_write_INT(CHAR_T *key, INT_T value)
{
    //TODO

    CHAR_T tmp_cmd[128] = {0};
    snprintf(tmp_cmd, 128, "mkdir -p /home/share/chuishi/Project/os_5.3/tuya.cfgs/;echo %d > /home/share/chuishi/Project/os_5.3/tuya.cfgs/%s", value, key);
    printf("write int exc: %s \r\n", tmp_cmd);
    system(tmp_cmd);
}

STATIC INT_T __tuya_app_read_INT(CHAR_T *key)
{
    //TODO

    CHAR_T tmp_file[64] = {0};
    snprintf(tmp_file, 64, "cat /home/share/chuishi/Project/os_5.3/tuya.cfgs/%s", key);
    printf("read int exc: %s \r\n", tmp_file);
    FILE *p_file = popen(tmp_file, "r");
    if(p_file == NULL)
    {
        printf("open fail.\r\n");
        return -1;
    }
    memset(tmp_file, 0, 64);
    fgets(tmp_file, 64, p_file);
    if (strlen(tmp_file) == 0)
    {
        printf("read fail.\r\n");
        return -1;
    }
    INT_T value = atoi(tmp_file);
    printf("readstr:%s andint:%d \r\n", tmp_file, value);

    pclose(p_file);

    return value;
}

/* delete file */
STATIC VOID ty_voice_delete_file(CHAR_T *filename)
{
    if (NULL == filename)
    {
        PR_ERR("filename is NULL");
        return;
    }

    CHAR_T tmp_cmd[256] = {0};
    snprintf(tmp_cmd, 256, "rm -rf %s", filename);
    system(tmp_cmd);
}

STATIC VOID ty_voice_dp_report_play_state(ty_voice_handle_s* phdl)
{
#ifdef TUYA_DP_VOICE_PLAY_STATE
    CHAR_T tmp_str[2] = {0};
    snprintf(tmp_str, 2, "%d", phdl->state);
    tuya_ipc_dp_report(NULL, TUYA_DP_VOICE_PLAY_STATE, PROP_ENUM, tmp_str, 1);
#endif
}

STATIC VOID ty_voice_dp_report_play_current(ty_voice_handle_s* phdl)
{
#ifdef TUYA_DP_VOICE_PLAY_CURRENT
    CHAR_T tmp_str[128] = {0};
    snprintf(tmp_str, 128, "{\"bizcode\":\"%s\",\"id\":%d,\"errcode\":0}", phdl->voice_code, phdl->play_id);
    tuya_ipc_dp_report(NULL, TUYA_DP_VOICE_PLAY_CURRENT, PROP_STR, tmp_str, 1);
#endif
}

STATIC VOID ty_voice_dp_report_play(ty_voice_handle_s* phdl, CHAR_T *status)
{
#ifdef TUYA_DP_VOICE_PLAY
    CHAR_T tmp_str[128] = {0};
    snprintf(tmp_str, 128, "{\"bizcode\":\"%s\",\"id\":%d,\"status\":\"%s\"}", phdl->voice_code, phdl->play_id, status);
    tuya_ipc_dp_report(NULL, TUYA_DP_VOICE_PLAY, PROP_STR, tmp_str, 1);
#endif
}

/* update voice list and remove voice files not in voice list */
/* if update is TRUE, update voice list immediately */
STATIC OPERATE_RET ty_voice_update_voice_list(ty_voice_handle_s* phdl, BOOL_T update)
{
    INT_T i;
    OPERATE_RET ret = OPRT_OK;

    if (!phdl->inited)
    {
        PR_ERR("ty_voice_demo not inited");
        return OPRT_COM_ERROR;
    }

    printf("ty_voice_update_voice_list\n");

    /* get voice list */
    ret = tuya_ipc_get_voice_list(phdl->voice_list, update);
    if (ret != 0)
    {
        PR_ERR("get voice list failed");
        return ret;
    }

    phdl->voice_num = 0;
    /* get voice number in voice list */
    for (i = 0; i < phdl->max_voice_num; i++)
    {
        if (phdl->voice_list[i].valid)
        {
            phdl->voice_num++;
        }
    }

    /* remove files not in voice list */
    DIR *dir = NULL;
    struct dirent *ptr = NULL;
    /* open voice diretory */
    dir = opendir(IPC_VOICE_FILE_PATH);
    if (NULL != dir)
    {
        while((ptr = readdir(dir)) != NULL)
        {
            if (0 == strcmp(ptr->d_name, ".") || 0 == strcmp(ptr->d_name, ".."))
            {
                continue;
            }

            /* keep file in voice list, delete other files */
            BOOL_T need_delete = TRUE;
            for (i = 0; i < phdl->voice_num; i++)
            {
                CHAR_T filename[IPC_WAV_FILENAME_LEN+1] = {0};
                if (strlen(phdl->voice_list[i].filename) > 0)
                {
                    snprintf(filename, IPC_WAV_FILENAME_LEN, "%s_%s.wav", phdl->voice_code, phdl->voice_list[i].filename);
                }
                else
                {
                    snprintf(filename, IPC_WAV_FILENAME_LEN, "%s_%d.wav", phdl->voice_code, phdl->voice_list[i].id);
                }

                if ((ptr->d_type == 8) && (!strcmp(filename, ptr->d_name)))
                {
                    PR_DEBUG("file %s is in voice list, keep it", filename);
                    need_delete = FALSE;
                    break;
                }
            }

            /* delete files not in voice list */
            if (need_delete)
            {
                CHAR_T play_file[IPC_WAV_FILENAME_LEN+1] = {0};
                CHAR_T delete_file[IPC_WAV_FILENAME_LEN+1] = {0};
                snprintf(delete_file, IPC_WAV_FILENAME_LEN, "%s/%s", IPC_VOICE_FILE_PATH, ptr->d_name);
                ty_voice_delete_file(delete_file);
            }
        }

        closedir(dir);
    }

    /* load stored play id */
    static BOOL_T first_load = TRUE;
    if (first_load)
    {
        INT_T play_id = __tuya_app_read_INT("tuya_play_id");
        PR_DEBUG("init play id: %d", play_id);
        for (i = 0; i < phdl->voice_num; i++)
        {
            if (play_id == phdl->voice_list[i].id)
            {
                PR_DEBUG("find play id: %d", play_id);
                phdl->play_index = i;
                phdl->play_id = play_id;
                break;
            }
        }
        phdl->play_id = phdl->voice_list[phdl->play_index].id;
        first_load = FALSE;

        ty_voice_dp_report_play_current(phdl);
    }

    tuya_ipc_get_utc_time(&phdl->voice_list_update_time);

    return OPRT_OK;
}

/* get index in voice list by id */
STATIC INT_T ty_voice_get_play_index(ty_voice_handle_s* phdl, CHAR_T *code, INT_T id)
{
    if (phdl->voice_num == 0)
    {
        PR_ERR("voice list is empty");
        return -1;
    }

    if (strcmp(code, phdl->voice_code))
    {
        PR_ERR("voice code %s is not match %s", code, phdl->voice_code);
        return -1;
    }

    INT_T i = 0;
    for (i = 0; i < phdl->voice_num; i++)
    {
        if (id == phdl->voice_list[i].id)
        {
            PR_DEBUG("find voice id %d", id);
            return i;
        }
    }

    return -1;
}

/**
 * \func: voice downloading callback, it will be called when voice data is received by TUYA SDK
 * \data: voice data
 * \len:  voice data length
 * \priv_data:  priv_data in tuya_ipc_voice_download_start interface
 */
STATIC OPERATE_RET ty_get_voice_data_cb(IN BYTE_T *data, IN UINT_T len, IN PVOID_T pri_data)
{
    ty_voice_handle_s* phdl = (ty_voice_handle_s *)pri_data;

    if (phdl->fp)
    {
        fwrite(data, 1, len, phdl->fp);
        fflush(phdl->fp);
    }

    return OPRT_OK;
}

/**
 * \func: voice download finish callback, it will be called when voice downloading finished
 * \download_result = 0: download success
 * \download_result > 0: download stop, if ty_voice_download_stop is called
 * \download_result < 0: download failed
 * \priv_data:  priv_data in tuya_ipc_voice_download_start interface
 */
STATIC VOID ty_voice_notify_cb(IN INT_T download_result, IN PVOID_T pri_data)
{
    ty_voice_handle_s* phdl = ty_voice_get_handle();

    if (phdl->fp)
    {
        fclose(phdl->fp);
        phdl->fp = NULL;
        sync();
        usleep(100 * 1000);
    }

    /* download success */
    pthread_mutex_lock(&phdl->lock);
    phdl->download_result = download_result;
    phdl->downloading = FALSE;
    pthread_mutex_unlock(&phdl->lock);
}

/* start downloading voice file */
STATIC INT_T ty_voice_download_start(ty_voice_handle_s* phdl)
{
    FILE *fp = NULL;
    OPERATE_RET ret;

    if (phdl->voice_num == 0)
    {
        return -1;
    }

    fp = fopen(phdl->play_filename, "w");
    if (NULL == fp)
    {
        PR_ERR("open file %s failed", phdl->play_filename);
        return -1;
    }
    phdl->fp = fp;

    ret = tuya_ipc_voice_download_start(phdl->voice_code, phdl->voice_list[phdl->play_index].id, ty_get_voice_data_cb, ty_voice_notify_cb, phdl);
    if (ret != 0)
    {
        PR_ERR("download file %s failed", phdl->play_filename);
        fclose(fp);
        phdl->fp = NULL;
        return -1;
    }

    return 0;
}

/* stop downloading voice file */
STATIC VOID ty_voice_download_stop()
{
    tuya_ipc_voice_download_stop();
}

/* format voice file name */
STATIC VOID ty_voice_format_filename(ty_voice_handle_s* phdl, CHAR_T *filename, INT_T play_id)
{
   memset(phdl->play_filename, 0, IPC_WAV_FILENAME_LEN);
   if (filename && strlen(filename) > 0)
   {
       snprintf(phdl->play_filename, IPC_WAV_FILENAME_LEN, "%s/%s_%s.wav", IPC_VOICE_FILE_PATH, phdl->voice_code, filename);
   }
   else
   {
       snprintf(phdl->play_filename, IPC_WAV_FILENAME_LEN, "%s/%s_%d.wav", IPC_VOICE_FILE_PATH, phdl->voice_code, play_id);
   }
}

/* switch to previous voice file */
STATIC VOID ty_voice_switch_to_prev(ty_voice_handle_s* phdl, IPC_VOICE_PLAY_MODE_E play_mode)
{
    if (phdl->voice_num == 0)
    {
        return;
    }

    /* get previous voice file */
    switch (play_mode)
    {
    case IPC_VOICE_PLAY_MODE_LOOP:
        if (phdl->play_index == 0)
        {
            phdl->play_index = phdl->voice_num - 1;
        }
        else
        {
            phdl->play_index = phdl->play_index - 1;
        }
        break;

    case IPC_VOICE_PLAY_MODE_SHUFFLE:
        phdl->play_index = rand() % phdl->voice_num;
        break;
    }

    phdl->play_id = phdl->voice_list[phdl->play_index].id;
    ty_voice_format_filename(phdl, phdl->voice_list[phdl->play_index].filename, phdl->play_id);
}

/* switch to next voice file */
STATIC VOID ty_voice_switch_to_next(ty_voice_handle_s* phdl, IPC_VOICE_PLAY_MODE_E play_mode)
{
    if (phdl->voice_num == 0) {
        return;
    }

    /* get next voice file */
    switch (play_mode)
    {
    case IPC_VOICE_PLAY_MODE_LOOP:
        if (phdl->play_index == phdl->voice_num - 1)
        {
            phdl->play_index = 0;
        }
        else
        {
            phdl->play_index = phdl->play_index + 1;
        }
        break;

    case IPC_VOICE_PLAY_MODE_SHUFFLE:
        phdl->play_index = rand() % phdl->voice_num;
        break;
    }

    phdl->play_id = phdl->voice_list[phdl->play_index].id;
    ty_voice_format_filename(phdl, phdl->voice_list[phdl->play_index].filename, phdl->play_id);
}

STATIC VOID ty_voice_status_process(ty_voice_handle_s *phdl, IPC_SPEAKER_STATUS_E speaker_status)
{
    switch (phdl->status)
    {
    case IPC_VOICE_STATUS_IDLE:
        if (phdl->state == IPC_VOICE_STATE_PLAYING)
        {
            phdl->status = IPC_VOICE_STATUS_READY;
        }
        break;

    case IPC_VOICE_STATUS_READY:
        if (!phdl->downloading)
        {
            if (ty_media_wav_check(phdl->play_filename) == 0)
            {
                phdl->status = IPC_VOICE_STATUS_PLAY_START;
            }
            else
            {
                phdl->download_fail_cnt = 0;
                phdl->status = IPC_VOICE_STATUS_DOWNLOAD_START;
            }
            /* report play current */
            ty_voice_dp_report_play_current(phdl);
            /* store play id */
            __tuya_app_write_INT("tuya_play_id", phdl->play_id);
        }
        break;

    case IPC_VOICE_STATUS_DOWNLOAD_START:
        if (phdl->download_fail_cnt >= IPC_VOICE_DOWNLOAD_RETRY)
        {
            PR_ERR("download fail cnt over limits %d", IPC_VOICE_DOWNLOAD_RETRY);
            phdl->status = IPC_VOICE_STATUS_STOP;
            phdl->state = IPC_VOICE_STATE_STOPPING;
            /* report download failed */
            ty_voice_dp_report_play(phdl, IPC_VOICE_STR_DOWNLOAD_FAIL);
            /* report play state: stopping */
            ty_voice_dp_report_play_state(phdl);
        }
        else
        {
            PR_DEBUG("start downloading file %s, retry cnt %d", phdl->play_filename, phdl->download_fail_cnt);
            if (phdl->download_fail_cnt == 0)
            {
                /* report downloading */
                ty_voice_dp_report_play(phdl, IPC_VOICE_STR_DOWNLOADING);
            }
            if (ty_voice_download_start(phdl) == 0)
            {
                phdl->downloading = TRUE;
                phdl->status = IPC_VOICE_STATUS_DOWNLOADING;
            }
            else
            {
                PR_ERR("downloading file %s failed", phdl->play_filename);
                phdl->download_fail_cnt++;
            }
        }
        break;

    case IPC_VOICE_STATUS_DOWNLOADING:
        if (!phdl->downloading)
        {
            /* download success */
            if (phdl->download_result == 0)
            {
                PR_DEBUG("downloading file %s success", phdl->play_filename);
                /* fail wav file check, stop playing */
                if (ty_media_wav_check(phdl->play_filename) != 0)
                {
                    PR_ERR("downloading file %s not complete", phdl->play_filename);
                    /* remove download file */
                    ty_voice_delete_file(phdl->play_filename);

                    phdl->download_fail_cnt = IPC_VOICE_DOWNLOAD_RETRY;
                    phdl->status = IPC_VOICE_STATUS_DOWNLOAD_START;
                }
                else
                {
                    phdl->status = IPC_VOICE_STATUS_PLAY_START;
                }
            }
            /* download stop */
            else if (phdl->download_result > 0)
            {
                PR_DEBUG("downloading file %s stop", phdl->play_filename);
                /* remove download file */
                ty_voice_delete_file(phdl->play_filename);
                phdl->status = IPC_VOICE_STATUS_STOP;
            }
            /* download fail */
            else
            {
                PR_DEBUG("downloading file %s failed", phdl->play_filename);
                /* remove download file */
                ty_voice_delete_file(phdl->play_filename);
                phdl->download_fail_cnt++;
                phdl->status = IPC_VOICE_STATUS_DOWNLOAD_START;
            }
        }
        break;

    case IPC_VOICE_STATUS_PLAY_START:
        PR_DEBUG("start play file %s", phdl->play_filename);
        /* play the voice file */
        ty_speaker_play_file(phdl->play_filename);
        phdl->status = IPC_VOICE_STATUS_PLAYING;
        /* report play state: playing */
        ty_voice_dp_report_play_state(phdl);
        break;

    case IPC_VOICE_STATUS_PLAYING:
        if (speaker_status == IPC_SPEAKER_STATUS_ERR)
        {
            PR_ERR("play file %s failed", phdl->play_filename);
            phdl->status = IPC_VOICE_STATUS_STOP;
            phdl->state = IPC_VOICE_STATE_STOPPING;
            /* report play failed */
            ty_voice_dp_report_play(phdl, IPC_VOICE_STR_PLAY_FAIL);
            /* report play state: stopping */
            ty_voice_dp_report_play_state(phdl);
        }
        else if (speaker_status == IPC_SPEAKER_STATUS_IDLE)
        {
            PR_DEBUG("play file %s finished", phdl->play_filename);
            ty_voice_switch_to_next(phdl, phdl->play_mode);
            phdl->status = IPC_VOICE_STATUS_READY;
        }
        break;

    case IPC_VOICE_STATUS_STOP:
        ty_speaker_stop_file();
        if (phdl->downloading)
        {
            ty_voice_download_stop();
        }
        phdl->status = IPC_VOICE_STATUS_IDLE;
        break;

    default:
        PR_ERR("unknown status: %d", phdl->status);
        break;
    }
}

STATIC void *ty_voice_demo_proc(void *arg)
{
    BOOL_T mqtt_status;
    BOOL_T mqtt_status_last;
    BOOL_T delay_dp_upload = FALSE;
    TIME_T current_time;
    IPC_SPEAKER_STATUS_E speaker_status;
    OPERATE_RET ret = OPRT_OK;
    ty_voice_handle_s* phdl = ty_voice_get_handle();
    
    PR_DEBUG("voice demo thread begin__");

    prctl(PR_SET_NAME, "voice_demo");

    while (phdl->inited)
    {
        mqtt_status = tuya_ipc_get_mqtt_status();
        /* mqtt is offline */
        if (0 == mqtt_status)
        {
            mqtt_status_last = mqtt_status;
            speaker_status = ty_speaker_get_status();
            pthread_mutex_lock(&phdl->lock);
            if (phdl->state != IPC_VOICE_STATE_STOPPING)
            {
                PR_ERR("mqtt is offline, stop playing voice");
                phdl->state = IPC_VOICE_STATE_STOPPING;
                phdl->status = IPC_VOICE_STATUS_STOP;
                ty_voice_status_process(phdl, speaker_status);
                delay_dp_upload = TRUE;
            }
            pthread_mutex_unlock(&phdl->lock);
            usleep(500 * 1000);
            continue;
        }
        else
        {
            /* device mqtt online */
            if (mqtt_status_last == FALSE)
            {
                mqtt_status_last = mqtt_status;
                /* report delayed play failed dp message */
                if (delay_dp_upload)
                {
                    delay_dp_upload = FALSE;
                    /* report play failed */
                    ty_voice_dp_report_play(phdl, IPC_VOICE_STR_PLAY_FAIL);
                    /* report play state: stopping */
                    ty_voice_dp_report_play_state(phdl);
                }
            }
        }

        pthread_mutex_lock(&phdl->lock);
        /* update voice list */
        tuya_ipc_get_utc_time(&current_time);
        if (phdl->voice_update)
        {
            ret = ty_voice_update_voice_list(phdl, TRUE);
        }
        else if (current_time - phdl->voice_list_update_time >= IPC_VOICE_LIST_UPDATE_DURATION)
        {
            ret = ty_voice_update_voice_list(phdl, FALSE);
        }

        /* reset update flag if update voice list failed */
        if (OPRT_OK != ret)
        {
            phdl->voice_update = TRUE;
        }
        else
        {
            phdl->voice_update = FALSE;
        }
    
        speaker_status = ty_speaker_get_status();
        ty_voice_status_process(phdl, speaker_status);
        pthread_mutex_unlock(&phdl->lock);

        usleep(300 * 1000);
    }

    PR_DEBUG("voice demo thread end__");
}

/* set play for TUYA_DP_VOICE_PLAY */
OPERATE_RET ty_voice_set_play(CHAR_T *code, INT_T id)
{
    INT_T index = 0;
    OPERATE_RET ret = OPRT_OK;
    ty_voice_handle_s* phdl = ty_voice_get_handle();

    if (NULL == code)
    {
        return OPRT_INVALID_PARM;
    }

    if (!phdl->inited)
    {
        PR_ERR("ty_voice_demo not inited");
        return OPRT_COM_ERROR;
    }

    pthread_mutex_lock(&phdl->lock);

    /* voice id not find in voice list */
    index = ty_voice_get_play_index(phdl, code, id);
    if (index < 0)
    {
        /* update voice list */
        ret = ty_voice_update_voice_list(phdl, TRUE);
        if (OPRT_OK != ret)
        {
            phdl->voice_update = TRUE;
            pthread_mutex_unlock(&phdl->lock);
            return OPRT_INVALID_PARM;
        }

        /* get index in voice list by id */
        index = ty_voice_get_play_index(phdl, code, id);
        if (index < 0)
        {
            PR_ERR("can not find voice id %d", id);
            pthread_mutex_unlock(&phdl->lock);
            return OPRT_INVALID_PARM;
        }
    }    

    phdl->play_index = index;
    phdl->play_id = phdl->voice_list[phdl->play_index].id;
    ty_voice_format_filename(phdl, phdl->voice_list[phdl->play_index].filename, phdl->play_id);
    phdl->state = IPC_VOICE_STATE_PLAYING;
    phdl->status = IPC_VOICE_STATUS_STOP;

    pthread_mutex_unlock(&phdl->lock);

    return OPRT_OK;
}

/* set play_control for TUYA_DP_VOICE_PLAY_CONTROL */
OPERATE_RET ty_voice_set_play_control(IPC_VOICE_PLAY_CONTROL_E cmd)
{
    ty_voice_handle_s* phdl = ty_voice_get_handle();

    if (!phdl->inited)
    {
        PR_ERR("ty_voice_demo not inited");
        return OPRT_COM_ERROR;
    }

    pthread_mutex_lock(&phdl->lock);

    if (phdl->voice_num == 0)
    {
        PR_ERR("voice list is empty");
        pthread_mutex_unlock(&phdl->lock);
        return OPRT_COM_ERROR;
    }

    switch (cmd)
    {
    case IPC_VOICE_PLAY_CTRL_PLAY:
        phdl->play_id = phdl->voice_list[phdl->play_index].id;
        ty_voice_format_filename(phdl, phdl->voice_list[phdl->play_index].filename, phdl->play_id);
        phdl->state = IPC_VOICE_STATE_PLAYING;
        phdl->status = IPC_VOICE_STATUS_STOP;
        break;

    case IPC_VOICE_PLAY_CTRL_STOP:
        phdl->state = IPC_VOICE_STATE_STOPPING;
        phdl->status = IPC_VOICE_STATUS_STOP;
        /* report play state: stopping */
        ty_voice_dp_report_play_state(phdl);
        break;

    case IPC_VOICE_PLAY_CTRL_NEXT:
        if (phdl->play_mode == IPC_VOICE_PLAY_MODE_LOOP1)
        {
            ty_voice_switch_to_next(phdl, IPC_VOICE_PLAY_MODE_LOOP);
        }
        else
        {
            ty_voice_switch_to_next(phdl, phdl->play_mode);
        }
        phdl->state = IPC_VOICE_STATE_PLAYING;
        phdl->status = IPC_VOICE_STATUS_STOP;
        break;

    case IPC_VOICE_PLAY_CTRL_PREV:
        if (phdl->play_mode == IPC_VOICE_PLAY_MODE_LOOP1)
        {
            ty_voice_switch_to_prev(phdl, IPC_VOICE_PLAY_MODE_LOOP);
        }
        else
        {
            ty_voice_switch_to_prev(phdl, phdl->play_mode);
        }
        phdl->state = IPC_VOICE_STATE_PLAYING;
        phdl->status = IPC_VOICE_STATUS_STOP;
        break;
    }

    pthread_mutex_unlock(&phdl->lock);

    return OPRT_OK;
}

/* get play_mode for TUYA_DP_VOICE_PLAY_MODE */
INT_T ty_voice_get_play_mode()
{
    INT_T play_mode = __tuya_app_read_INT("tuya_play_mode");

    if (play_mode < 0)
    {
        play_mode = IPC_VOICE_PLAY_MODE_LOOP;
        __tuya_app_write_INT("tuya_play_mode", play_mode);
    }

    return play_mode;
}

/* set play_mode for TUYA_DP_VOICE_PLAY_MODE */
OPERATE_RET ty_voice_set_play_mode(IPC_VOICE_PLAY_MODE_E mode)
{
    ty_voice_handle_s* phdl = ty_voice_get_handle();

    if (!phdl->inited)
    {
        PR_ERR("ty_voice_demo not inited");
        return OPRT_COM_ERROR;
    }

    pthread_mutex_lock(&phdl->lock);

    switch (mode)
    {    
    case IPC_VOICE_PLAY_MODE_LOOP:
        phdl->play_mode = IPC_VOICE_PLAY_MODE_LOOP;
        break;

    case IPC_VOICE_PLAY_MODE_LOOP1:
        phdl->play_mode = IPC_VOICE_PLAY_MODE_LOOP1;
        break;

    case IPC_VOICE_PLAY_MODE_SHUFFLE:
        phdl->play_mode = IPC_VOICE_PLAY_MODE_SHUFFLE;
        break;
    }

    pthread_mutex_unlock(&phdl->lock);

    __tuya_app_write_INT("tuya_play_mode", phdl->play_mode);

    return OPRT_OK;
}

/* get play_volume for TUYA_DP_VOICE_PLAY_VOLUME */
INT_T ty_voice_get_play_volume()
{
    INT_T volume = __tuya_app_read_INT("tuya_play_volume");

    if (volume < 0 || volume > 100)
    {
        volume = IPC_VOICE_DEFAULT_VOLUME;
        __tuya_app_write_INT("tuya_play_volume", volume);
    }

    return volume;
}

/* set play_volume for TUYA_DP_VOICE_PLAY_VOLUME */
OPERATE_RET ty_voice_set_play_volume(INT_T volume)
{
    ty_voice_handle_s* phdl = ty_voice_get_handle();

    if (!phdl->inited)
    {
        PR_ERR("ty_voice_demo not inited");
        return OPRT_COM_ERROR;
    }

    if (volume < 0 || volume > 100)
    {
        PR_ERR("volume %d is out of range: 0 ~ 100", volume);
        return OPRT_INVALID_PARM;
    }

    //ty_speaker_set_volume(volume);

    __tuya_app_write_INT("tuya_play_volume", volume);

    return OPRT_OK;
}

/* update voice list for TUYA_DP_VOICE_UPGRADE */
OPERATE_RET ty_voice_set_update_voice_list()
{
   ty_voice_handle_s* phdl = ty_voice_get_handle();

    if (!phdl->inited)
    {
        PR_ERR("ty_voice_demo not inited");
        return OPRT_COM_ERROR;
    }

    pthread_mutex_lock(&phdl->lock);
    phdl->voice_update = TRUE;
    pthread_mutex_unlock(&phdl->lock);

    return OPRT_OK;
}

/* get play_state for TUYA_DP_VOICE_PLAY_STATE */
INT_T ty_voice_get_play_state()
{
    INT_T state = IPC_VOICE_STATE_STOPPING;
    ty_voice_handle_s* phdl = ty_voice_get_handle();

    if (!phdl->inited)
    {
        PR_ERR("ty_voice_demo not inited");
        return state;
    }

    pthread_mutex_lock(&phdl->lock);
    state = phdl->state;
    pthread_mutex_unlock(&phdl->lock);

    return state;
}

OPERATE_RET TUYA_APP_Enable_Voice()
{
    OPERATE_RET ret;
    UINT_T max_voice_num = 30;
    ty_voice_handle_s* phdl = ty_voice_get_handle();

    if (phdl->inited)
    {
        PR_ERR("ty_voice_demo already inited");
        return -1;
    }

    /* check voice file directory */
    DIR *dir = NULL;
    dir = opendir(IPC_VOICE_FILE_PATH);
    if (NULL == dir)
    {
        PR_ERR("directory %s is not exist, please create first", IPC_VOICE_FILE_PATH);
        return OPRT_COM_ERROR;
    }
    closedir(dir);

    /* init voice */
    ret = tuya_ipc_voice_init(IPC_VOICE_CODE, max_voice_num);
    if (ret != OPRT_OK)
    {
        PR_ERR("tuya_ipc_voice_init failed");
        return ret;
    }

    /* malloc buffer for voice list */
    strncpy(phdl->voice_code, IPC_VOICE_CODE, IPC_VOICE_CODE_LEN);
    phdl->voice_list = (VOICE_LIST_S *)malloc(sizeof(VOICE_LIST_S) * max_voice_num);
    if (phdl->voice_list == NULL)
    {
        PR_ERR("malloc for voice list failed");
        return OPRT_MALLOC_FAILED;
    }
    memset(phdl->voice_list, 0, sizeof(VOICE_LIST_S) * max_voice_num);
    phdl->max_voice_num = max_voice_num;
    phdl->voice_num = 0;
    phdl->voice_list_update_time = 0;

    /* init play control parameters */
    phdl->state = IPC_VOICE_STATE_STOPPING;
    phdl->status = IPC_VOICE_STATUS_IDLE;
    phdl->play_index = 0;
    phdl->play_id = 0;
    memset(phdl->play_filename, 0, sizeof(phdl->play_filename));

    /* get stored play mode */
    phdl->play_mode = ty_voice_get_play_mode();

    /* get stored play volume */
    INT_T volume = ty_voice_get_play_volume();
    ty_speaker_set_volume(volume);

    /* create play control thread */
    pthread_attr_t attr;
    ret = pthread_attr_init(&attr);
    if (ret != 0)
    {
        PR_ERR("pthread_attr_init failed ret[%d]", ret);
        free(phdl->voice_list);
        return ret;
    }
    pthread_attr_setstacksize(&attr, 64 * 1024);
    ret = pthread_create(&phdl->stid, &attr, ty_voice_demo_proc, NULL);
    pthread_attr_destroy(&attr);
    if (ret != 0)
    {
        PR_ERR("speaker control thread create failed ret[%d]", ret);
        free(phdl->voice_list);
        return ret;
    }

    pthread_detach(phdl->stid);

    phdl->inited = TRUE;

    return OPRT_OK;
}
#endif
